@@ -1,25 +1,9 @@
-# -*- Mode: perl; indent-tabs-mode: nil -*-
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at http://mozilla.org/MPL/2.0/.
 #
-# The contents of this file are subject to the Mozilla Public
-# License Version 1.1 (the "License"); you may not use this file
-# except in compliance with the License. You may obtain a copy of
-# the License at http://www.mozilla.org/MPL/
-#
-# Software distributed under the License is distributed on an "AS
-# IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
-# implied. See the License for the specific language governing
-# rights and limitations under the License.
-#
-# The Original Code is the Bugzilla Bug Tracking System.
-#
-# The Initial Developer of the Original Code is Netscape Communications
-# Corporation. Portions created by Netscape are
-# Copyright (C) 1998 Netscape Communications Corporation. All
-# Rights Reserved.
-#
-# Contributor(s): Gervase Markham <gerv@gerv.net>
-#                 Albert Ting <altlst@sonic.net>
-#                 A. Karl Kornel <karl@kornel.name>
+# This Source Code Form is "Incompatible With Secondary Licenses", as
+# defined by the Mozilla Public License, v. 2.0.
 
 use strict;
 
@@ -67,7 +51,7 @@ sub init {
     #
     # The URL encoding is:
     # line0=67&line0=73&line1=81&line2=67...
-    # &label0=B+/+R+/+NEW&label1=...
+    # &label0=B+/+R+/+CONFIRMED&label1=...
     # &select0=1&select3=1...    
     # &cumulate=1&datefrom=2002-02-03&dateto=2002-04-04&ctype=html...
     # &gt=1&labelgt=Grand+Total    
@@ -437,8 +421,8 @@ sub dump {
     my $data = $self->data;
     
     require Data::Dumper;
-    print "<pre>Bugzilla::Chart object:\n";
-    print Data::Dumper::Dumper($self);
+    say "<pre>Bugzilla::Chart object:";
+    print html_quote(Data::Dumper::Dumper($self));
     print "</pre>";
 }
 
