@@ -1,27 +1,13 @@
-# -*- Mode: perl; indent-tabs-mode: nil -*-
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at http://mozilla.org/MPL/2.0/.
 #
-# The contents of this file are subject to the Mozilla Public
-# License Version 1.1 (the "License"); you may not use this file
-# except in compliance with the License. You may obtain a copy of
-# the License at http://www.mozilla.org/MPL/
-#
-# Software distributed under the License is distributed on an "AS
-# IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
-# implied. See the License for the specific language governing
-# rights and limitations under the License.
-#
-# The Original Code is the Bugzilla Bug Tracking System.
-#
-# Contributor(s): Tiago R. Mello <timello@async.com.br>
-#                 Frédéric Buclin <LpSolit@gmail.com>
-#                 Max Kanat-Alexander <mkanat@bugzilla.org>
-#                 Akamai Technologies <bugzilla-dev@akamai.com>
-
-use strict;
+# This Source Code Form is "Incompatible With Secondary Licenses", as
+# defined by the Mozilla Public License, v. 2.0.
 
 package Bugzilla::Component;
-
-use base qw(Bugzilla::Object);
+use strict;
+use base qw(Bugzilla::Field::ChoiceInterface Bugzilla::Object);
 
 use Bugzilla::Constants;
 use Bugzilla::Util;
@@ -30,11 +16,15 @@ use Bugzilla::User;
 use Bugzilla::FlagType;
 use Bugzilla::Series;
 
+use Scalar::Util qw(blessed);
+
 ###############################
 ####    Initialization     ####
 ###############################
 
 use constant DB_TABLE => 'components';
+# This is mostly for the editfields.cgi case where ->get_all is called.
+use constant LIST_ORDER => 'product_id, name';
 
 use constant DB_COLUMNS => qw(
     id
@@ -43,13 +33,7 @@ use constant DB_COLUMNS => qw(
     initialowner
     initialqacontact
     description
-);
-
-use constant REQUIRED_CREATE_FIELDS => qw(
-    name
-    product
-    initialowner
-    description
+    isactive
 );
 
 use constant UPDATE_COLUMNS => qw(
@@ -57,8 +41,13 @@ use constant UPDATE_COLUMNS => qw(
     initialowner
     initialqacontact
     description
+    isactive
 );
 
+use constant REQUIRED_FIELD_MAP => {
+    product_id => 'product',
+};
+
 use constant VALIDATORS => {
     create_series    => \&Bugzilla::Object::check_boolean,
     product          => \&_check_product,
@@ -66,10 +55,12 @@ use constant VALIDATORS => {
     initialqacontact => \&_check_initialqacontact,
     description      => \&_check_description,
     initial_cc       => \&_check_cc_list,
+    name             => \&_check_name,
+    isactive         => \&Bugzilla::Object::check_boolean,
 };
 
-use constant UPDATE_VALIDATORS => {
-    name => \&_check_name,
+use constant VALIDATOR_DEPENDENCIES => {
+    name => ['product'],
 };
 
 ###############################
@@ -80,7 +71,7 @@ sub new {
     my $dbh = Bugzilla->dbh;
 
     my $product;
-    if (ref $param) {
+    if (ref $param and !defined $param->{id}) {
         $product = $param->{product};
         my $name = $param->{name};
         if (!defined $product) {
@@ -116,8 +107,11 @@ sub create {
     my $params = $class->run_create_validators(@_);
     my $cc_list = delete $params->{initial_cc};
     my $create_series = delete $params->{create_series};
+    my $product = delete $params->{product};
+    $params->{product_id} = $product->id;
 
     my $component = $class->insert_create_data($params);
+    $component->{product} = $product;
 
     # We still have to fill the component_cc table.
     $component->_update_cc_list($cc_list) if $cc_list;
@@ -129,17 +123,6 @@ sub create {
     return $component;
 }
 
-sub run_create_validators {
-    my $class  = shift;
-    my $params = $class->SUPER::run_create_validators(@_);
-
-    my $product = delete $params->{product};
-    $params->{product_id} = $product->id;
-    $params->{name} = $class->_check_name($params->{name}, $product);
-
-    return $params;
-}
-
 sub update {
     my $self = shift;
     my $changes = $self->SUPER::update(@_);
@@ -156,8 +139,15 @@ sub remove_from_db {
     my $self = shift;
     my $dbh = Bugzilla->dbh;
 
+    $self->_check_if_controller(); # From ChoiceInterface
+
     $dbh->bz_start_transaction();
 
+    # Products must have at least one component.
+    if (scalar(@{$self->product->components}) == 1) {
+        ThrowUserError('component_is_last', { comp => $self });
+    }
+
     if ($self->bug_count) {
         if (Bugzilla->params->{'allowbugdeletion'}) {
             require Bugzilla::Bug;
@@ -171,14 +161,7 @@ sub remove_from_db {
             ThrowUserError('component_has_bugs', {nb => $self->bug_count});
         }
     }
-
-    $dbh->do('DELETE FROM flaginclusions WHERE component_id = ?',
-             undef, $self->id);
-    $dbh->do('DELETE FROM flagexclusions WHERE component_id = ?',
-             undef, $self->id);
-    $dbh->do('DELETE FROM component_cc WHERE component_id = ?',
-             undef, $self->id);
-    $dbh->do('DELETE FROM components WHERE id = ?', undef, $self->id);
+    $self->SUPER::remove_from_db();
 
     $dbh->bz_commit_transaction();
 }
@@ -188,7 +171,8 @@ sub remove_from_db {
 ################################
 
 sub _check_name {
-    my ($invocant, $name, $product) = @_;
+    my ($invocant, $name, undef, $params) = @_;
+    my $product = blessed($invocant) ? $invocant->product : $params->{product};
 
     $name = trim($name);
     $name || ThrowUserError('component_blank_name');
@@ -197,7 +181,6 @@ sub _check_name {
         ThrowUserError('component_name_too_long', {'name' => $name});
     }
 
-    $product = $invocant->product if (ref $invocant);
     my $component = new Bugzilla::Component({product => $product, name => $name});
     if ($component && (!ref $invocant || $component->id != $invocant->id)) {
         ThrowUserError('component_already_exists', { name    => $component->name,
@@ -237,6 +220,8 @@ sub _check_initialqacontact {
 
 sub _check_product {
     my ($invocant, $product) = @_;
+    $product || ThrowCodeError('param_required', 
+                    { function => "$invocant->create", param => 'product' });
     return Bugzilla->user->check_can_admin_product($product->name);
 }
 
@@ -304,6 +289,7 @@ sub _create_series {
 
 sub set_name { $_[0]->set('name', $_[1]); }
 sub set_description { $_[0]->set('description', $_[1]); }
+sub set_is_active { $_[0]->set('isactive', $_[1]); }
 sub set_default_assignee {
     my ($self, $owner) = @_;
 
@@ -363,6 +349,8 @@ sub default_assignee {
 sub default_qa_contact {
     my $self = shift;
 
+    return if !$self->{'initialqacontact'};
+
     if (!defined $self->{'default_qa_contact'}) {
         $self->{'default_qa_contact'} =
             new Bugzilla::User($self->{'initialqacontact'});
@@ -374,16 +362,14 @@ sub flag_types {
     my $self = shift;
 
     if (!defined $self->{'flag_types'}) {
+        my $flagtypes = Bugzilla::FlagType::match({ product_id   => $self->product_id,
+                                                    component_id => $self->id });
+
         $self->{'flag_types'} = {};
         $self->{'flag_types'}->{'bug'} =
-          Bugzilla::FlagType::match({ 'target_type'  => 'bug',
-                                      'product_id'   => $self->product_id,
-                                      'component_id' => $self->id });
-
+          [grep { $_->target_type eq 'bug' } @$flagtypes];
         $self->{'flag_types'}->{'attachment'} =
-          Bugzilla::FlagType::match({ 'target_type'  => 'attachment',
-                                      'product_id'   => $self->product_id,
-                                      'component_id' => $self->id });
+          [grep { $_->target_type eq 'attachment' } @$flagtypes];
     }
     return $self->{'flag_types'};
 }
@@ -418,10 +404,24 @@ sub product {
 ####      Accessors        ####
 ###############################
 
-sub id          { return $_[0]->{'id'};          }
-sub name        { return $_[0]->{'name'};        }
 sub description { return $_[0]->{'description'}; }
 sub product_id  { return $_[0]->{'product_id'};  }
+sub is_active   { return $_[0]->{'isactive'};    }
+
+##############################################
+# Implement Bugzilla::Field::ChoiceInterface #
+##############################################
+
+use constant FIELD_NAME => 'component';
+use constant is_default => 0;
+
+sub is_set_on_bug {
+    my ($self, $bug) = @_;
+    my $value = blessed($bug) ? $bug->component_id : $bug->{component};
+    $value = $value->id if blessed($value);
+    return 0 unless $value;
+    return $value == $self->id ? 1 : 0;
+}
 
 ###############################
 ####      Subroutines      ####
@@ -489,10 +489,12 @@ Component.pm represents a Product Component object.
 
  Params:      $param - If you pass an integer, the integer is the
                        component ID from the database that we want to
-                       read in. If you pass in a hash with the 'name'
-                       and 'product' keys, then the value of the name
-                       key is the name of a component being in the given
-                       product.
+                       read in. 
+                       However, If you pass in a hash, it must contain
+                       two keys:
+                       name (string): the name of the component
+                       product (object): an object of Bugzilla::Product
+                       representing the product that the component belongs to.
 
  Returns:     A Bugzilla::Component object.
 
@@ -504,7 +506,7 @@ Component.pm represents a Product Component object.
 
  Returns:     Integer with the number of bugs.
 
-=item C<bugs_ids()>
+=item C<bug_ids()>
 
  Description: Returns all bug IDs that belong to the component.
 
@@ -528,7 +530,8 @@ Component.pm represents a Product Component object.
 
  Params:      none.
 
- Returns:     A Bugzilla::User object.
+ Returns:     A Bugzilla::User object if the default QA contact is defined for
+              the component. Otherwise, returns undef.
 
 =item C<initial_cc>
 
@@ -630,17 +633,17 @@ Component.pm represents a Product Component object.
  Description: Create a new component for the given product.
 
  Params:      The hashref must have the following keys:
-              name            - name of the new component (string). This name
-                                must be unique within the product.
-              product         - a Bugzilla::Product object to which
-                                the Component is being added.
-              description     - description of the new component (string).
-              initialowner    - login name of the default assignee (string).
+              name             - name of the new component (string). This name
+                                 must be unique within the product.
+              product          - a Bugzilla::Product object to which
+                                 the Component is being added.
+              description      - description of the new component (string).
+              initialowner     - login name of the default assignee (string).
               The following keys are optional:
-              initiaqacontact - login name of the default QA contact (string),
-                                or an empty string to clear it.
-              initial_cc      - an arrayref of login names to add to the
-                                CC list by default.
+              initialqacontact - login name of the default QA contact (string),
+                                 or an empty string to clear it.
+              initial_cc       - an arrayref of login names to add to the
+                                 CC list by default.
 
  Returns:     A Bugzilla::Component object.
 
