@@ -1,22 +1,9 @@
-# -*- Mode: perl; indent-tabs-mode: nil -*-
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at http://mozilla.org/MPL/2.0/.
 #
-# The contents of this file are subject to the Mozilla Public
-# License Version 1.1 (the "License"); you may not use this file
-# except in compliance with the License. You may obtain a copy of
-# the License at http://www.mozilla.org/MPL/
-#
-# Software distributed under the License is distributed on an "AS
-# IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
-# implied. See the License for the specific language governing
-# rights and limitations under the License.
-#
-# The Original Code is the Bugzilla Bug Tracking System.
-#
-# The Initial Developer of the Original Code is Everything Solved, Inc.
-# Portions created by Everything Solved are Copyright (C) 2007
-# Everything Solved, Inc. All Rights Reserved.
-#
-# Contributor(s): Max Kanat-Alexander <mkanat@bugzilla.org>
+# This Source Code Form is "Incompatible With Secondary Licenses", as
+# defined by the Mozilla Public License, v. 2.0.
 
 package Bugzilla::Install::CPAN;
 use strict;
@@ -33,10 +20,10 @@ use Bugzilla::Constants;
 use Bugzilla::Install::Requirements qw(have_vers);
 use Bugzilla::Install::Util qw(bin_loc install_string);
 
+use Config;
 use CPAN;
 use Cwd qw(abs_path);
 use File::Path qw(rmtree);
-use List::Util qw(shuffle);
 
 # These are required for install-module.pl to be able to install
 # all modules properly.
@@ -54,6 +41,13 @@ use constant REQUIREMENTS => (
         package => 'YAML',
         version => 0,
     },
+    {
+        # Many modules on CPAN are now built with Dist::Zilla, which
+        # unfortunately means they require this version of EU::MM to install.
+        module  => 'ExtUtils::MakeMaker',
+        package => 'ExtUtils-MakeMaker',
+        version => '6.31',
+    },
 );
 
 # We need the absolute path of ext_libpath, because CPAN chdirs around
@@ -73,6 +67,8 @@ use constant CPAN_DEFAULTS => {
     build_cache => 0,
     build_requires_install_policy => 'yes',
     cache_metadata => 1,
+    colorize_output => 1,
+    colorize_print => 'bold',
     index_expire => 1,
     scan_cache => 'atstart',
 
@@ -89,17 +85,14 @@ use constant CPAN_DEFAULTS => {
     unzip => bin_loc('unzip'),
     wget => bin_loc('wget'),
 
-    urllist => [shuffle qw(
-        http://cpan.pair.com/
-        http://mirror.hiwaay.net/CPAN/
-        ftp://ftp.dc.aleron.net/pub/CPAN/
-        http://mirrors.kernel.org/cpan/
-        http://mirrors2.kernel.org/cpan/)],
+    urllist => ['http://www.cpan.org/'],
 };
 
 sub check_cpan_requirements {
     my ($original_dir, $original_args) = @_;
 
+    _require_compiler();
+
     my @install;
     foreach my $module (REQUIREMENTS) {
         my $installed = have_vers($module, 1);
@@ -120,6 +113,26 @@ sub check_cpan_requirements {
     }
 }
 
+sub _require_compiler {
+    my @errors;
+
+    my $cc_name = $Config{cc};
+    my $cc_exists = bin_loc($cc_name);
+
+    if (!$cc_exists) {
+        push(@errors, install_string('install_no_compiler'));
+    }
+
+    my $make_name = $CPAN::Config->{make};
+    my $make_exists = bin_loc($make_name);
+
+    if (!$make_exists) {
+        push(@errors, install_string('install_no_make'));
+    }
+
+    die @errors if @errors;
+}
+
 sub install_module {
     my ($name, $test) = @_;
     my $bzlib = BZ_LIB;
@@ -143,17 +156,26 @@ sub install_module {
     elsif ($name eq 'SOAP::Lite') {
         $CPAN::Config->{makepl_arg} .= " --noprompt";
     }
-    # MIME-tools has a Module::Install that's too old to understand alldeps.
-    elsif ($name =~ /^MIME::/) {
-        $ENV{PERL_AUTOINSTALL} = '--defaultdeps';
-    }
 
     my $module = CPAN::Shell->expand('Module', $name);
     if (!$module) {
         die install_string('no_such_module', { module => $name }) . "\n";
     }
+    my $version = $module->cpan_version;
+    my $module_name = $name;
+
+    if ($name eq 'LWP::UserAgent' && $^V lt v5.8.8) {
+        # LWP 6.x requires Perl 5.8.8 or newer.
+        # As PAUSE only indexes the very last version of each module,
+        # we have to specify the path to the tarball ourselves.
+        $name = 'GAAS/libwww-perl-5.837.tar.gz';
+        # This tarball contains LWP::UserAgent 5.835.
+        $version = '5.835';
+    }
+
     print install_string('install_module', 
-              { module => $name, version => $module->cpan_version }) . "\n";
+              { module => $module_name, version => $version }) . "\n";
+
     if ($test) {
         CPAN::Shell->force('install', $name);
     }
@@ -196,7 +218,7 @@ sub set_cpan_config {
 
         # If we can't make one, we finally try to use the Bugzilla directory.
         if (!-w $dir) {
-            print "WARNING: Using the Bugzilla directory as the CPAN home.\n";
+            print STDERR install_string('cpan_bugzilla_home'), "\n";
             $dir = "$bzlib/.cpan";
         }
     }
