@@ -1,19 +1,9 @@
-# -*- Mode: perl; indent-tabs-mode: nil -*-
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at http://mozilla.org/MPL/2.0/.
 #
-# The contents of this file are subject to the Mozilla Public
-# License Version 1.1 (the "License"); you may not use this file
-# except in compliance with the License. You may obtain a copy of
-# the License at http://www.mozilla.org/MPL/
-#
-# Software distributed under the License is distributed on an "AS
-# IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
-# implied. See the License for the specific language governing
-# rights and limitations under the License.
-#
-# The Original Code is the Bugzilla Bug Tracking System.
-#
-# Contributor(s): Max Kanat-Alexander <mkanat@bugzilla.org>
-#                 Marc Schumann <wurblzap@gmail.com>
+# This Source Code Form is "Incompatible With Secondary Licenses", as
+# defined by the Mozilla Public License, v. 2.0.
 
 package Bugzilla::Install::Requirements;
 
@@ -24,14 +14,21 @@ package Bugzilla::Install::Requirements;
 # MUST NOT "use."
 
 use strict;
+use version;
 
 use Bugzilla::Constants;
-use Bugzilla::Install::Util qw(vers_cmp install_string 
+use Bugzilla::Install::Util qw(vers_cmp install_string bin_loc 
                                extension_requirement_packages);
 use List::Util qw(max);
-use Safe;
 use Term::ANSIColor;
 
+# Return::Value 1.666002 pollutes the error log with warnings about this
+# deprecated module. We have to set NO_CLUCK = 1 before loading Email::Send
+# in have_vers() to disable these warnings.
+BEGIN {
+    $Return::Value::NO_CLUCK = 1;
+}
+
 use base qw(Exporter);
 our @EXPORT = qw(
     REQUIRED_MODULES
@@ -49,6 +46,34 @@ our @EXPORT = qw(
 # by checksetup.pl.
 use constant TABLE_WIDTH => 71;
 
+# Optional Apache modules that have no Perl component to them.
+# If these are installed, Bugzilla has additional functionality.
+#
+# The keys are the names of the modules, the values are what the module
+# is called in the output of "apachectl -t -D DUMP_MODULES".
+use constant APACHE_MODULES => { 
+    mod_headers => 'headers_module',
+    mod_env     => 'env_module',
+    mod_expires => 'expires_module',
+};
+
+# These are all of the binaries that we could possibly use that can
+# give us info about which Apache modules are installed.
+# If we can't use "apachectl", the "httpd" binary itself takes the same
+# parameters. Note that on Debian and Gentoo, there is an "apache2ctl",
+# but it takes different parameters on each of those two distros, so we
+# don't use apache2ctl.
+use constant APACHE => qw(apachectl httpd apache2 apache);
+
+# If we don't find any of the above binaries in the normal PATH,
+# these are extra places we look.
+use constant APACHE_PATH => [qw(
+    /usr/sbin 
+    /usr/local/sbin
+    /usr/libexec
+    /usr/local/libexec
+)];
+
 # The below two constants are subroutines so that they can implement
 # a hook. Other than that they are actually constants.
 
@@ -66,22 +91,20 @@ sub REQUIRED_MODULES {
     {
         package => 'CGI.pm',
         module  => 'CGI',
-        # Perl 5.10 requires CGI 3.33 due to a taint issue when
-        # uploading attachments, see bug 416382.
-        # Require CGI 3.21 for -httponly support, see bug 368502.
-        version => (vers_cmp($perl_ver, '5.10') > -1) ? '3.33' : '3.21',
-        # CGI::Carp in 3.46 and 3.47 breaks Template Toolkit
-        blacklist => ['^3\.46$', '^3\.47$'],
+        # 3.51 fixes a security problem that affects Bugzilla.
+        # (bug 591165)
+        version => '3.51',
     },
     {
         package => 'Digest-SHA',
         module  => 'Digest::SHA',
         version => 0
     },
+    # 0.23 fixes incorrect handling of 1/2 & 3/4 timezones.
     {
         package => 'TimeDate',
         module  => 'Date::Format',
-        version => '2.21'
+        version => '2.23'
     },
     # 0.28 fixed some important bugs in DateTime.
     {
@@ -98,10 +121,11 @@ sub REQUIRED_MODULES {
         module  => 'DateTime::TimeZone',
         version => ON_WINDOWS ? '0.79' : '0.71'
     },
+    # 1.54 is required for Perl 5.10+. It also makes DBD::Oracle happy.
     {
         package => 'DBI',
         module  => 'DBI',
-        version => '1.41'
+        version => (vers_cmp($perl_ver, '5.13.3') > -1) ? '1.614' : '1.54'
     },
     # 2.22 fixes various problems related to UTF8 strings in hash keys,
     # as well as line endings on Windows.
@@ -110,41 +134,61 @@ sub REQUIRED_MODULES {
         module  => 'Template',
         version => '2.22'
     },
+    # 2.04 implement the "Test" method (to write to data/mailer.testfile).
     {
         package => 'Email-Send',
         module  => 'Email::Send',
-        version => ON_WINDOWS ? '2.16' : '2.00',
+        version => ON_WINDOWS ? '2.16' : '2.04',
         blacklist => ['^2\.196$']
     },
     {
         package => 'Email-MIME',
         module  => 'Email::MIME',
-        version => '1.861'
-    },
-    {
-        package => 'Email-MIME-Encodings',
-        module  => 'Email::MIME::Encodings',
-        # Fixes bug 486206
-        version => '1.313',
-    },
-    {
-        package => 'Email-MIME-Modifier',
-        module  => 'Email::MIME::Modifier',
-        version => '1.442'
+        # This fixes a memory leak in walk_parts that affected jobqueue.pl.
+        version => '1.904'
     },
     {
         package => 'URI',
         module  => 'URI',
-        version => 0
+        # This version properly handles a semicolon as the delimiter
+        # in a URL query string.
+        version => '1.37',
+    },
+    # 0.32 fixes several memory leaks in the XS version of some functions.
+    {
+        package => 'List-MoreUtils',
+        module  => 'List::MoreUtils',
+        version => 0.32,
+    },
+    {
+        package => 'Math-Random-ISAAC',
+        module  => 'Math::Random::ISAAC',
+        version => '1.0.1',
     },
     );
 
+    if (ON_WINDOWS) {
+        push(@modules, {
+            package => 'Win32',
+            module  => 'Win32',
+            # 0.35 fixes a memory leak in GetOSVersion, which we use.
+            version => 0.35,
+        }, 
+        {
+            package => 'Win32-API',
+            module  => 'Win32::API',
+            # 0.55 fixes a bug with char* that might affect Bugzilla::RNG.
+            version => '0.55',
+        });
+    }
+
     my $extra_modules = _get_extension_requirements('REQUIRED_MODULES');
     push(@modules, @$extra_modules);
     return \@modules;
 };
 
 sub OPTIONAL_MODULES {
+    my $perl_ver = sprintf('%vd', $^V);
     my @modules = (
     {
         package => 'GD',
@@ -156,7 +200,9 @@ sub OPTIONAL_MODULES {
         package => 'Chart',
         module  => 'Chart::Lines',
         # Versions below 2.1 cannot be detected accurately.
-        version => '2.1',
+        # There is no 2.1.0 release (it was 2.1), but .0 is required to fix
+        # https://rt.cpan.org/Public/Bug/Display.html?id=28218.
+        version => '2.1.0',
         feature => [qw(new_charts old_charts)],
     },
     {
@@ -179,12 +225,6 @@ sub OPTIONAL_MODULES {
         version => 0,
         feature => ['graphical_reports'],
     },
-    {
-        package => 'XML-Twig',
-        module  => 'XML::Twig',
-        version => 0,
-        feature => ['moving', 'updates'],
-    },
     {
         package => 'MIME-tools',
         # MIME::Parser is packaged as MIME::Tools on ActiveState Perl
@@ -198,10 +238,17 @@ sub OPTIONAL_MODULES {
         version => 0,
         feature => ['updates'],
     },
+    {
+        package => 'XML-Twig',
+        module  => 'XML::Twig',
+        version => 0,
+        feature => ['moving', 'updates'],
+    },
     {
         package => 'PatchReader',
         module  => 'PatchReader',
-        version => '0.9.4',
+        # 0.9.6 fixes two notable bugs and significantly improves the UX.
+        version => '0.9.6',
         feature => ['patch_viewer'],
     },
     {
@@ -216,18 +263,34 @@ sub OPTIONAL_MODULES {
         version => 0,
         feature => ['smtp_auth'],
     },
+    {
+        package => 'Net-SMTP-SSL',
+        module  => 'Net::SMTP::SSL',
+        version => 1.01,
+        feature => ['smtp_ssl'],
+    },
     {
         package => 'RadiusPerl',
         module  => 'Authen::Radius',
         version => 0,
         feature => ['auth_radius'],
     },
+    # XXX - Once we require XMLRPC::Lite 0.717 or higher, we can
+    # remove SOAP::Lite from the list.
     {
         package => 'SOAP-Lite',
         module  => 'SOAP::Lite',
-        # 0.710.04 is required for correct UTF-8 handling, but .04 and .05 are
-        # affected by bug 468009.
-        version => '0.710.06',
+        # Fixes various bugs, including 542931 and 552353 + stops
+        # throwing warnings with Perl 5.12.
+        version => '0.712',
+        feature => ['xmlrpc'],
+    },
+    # Since SOAP::Lite 1.0, XMLRPC::Lite is no longer included
+    # and so it must be checked separately.
+    {
+        package => 'XMLRPC-Lite',
+        module  => 'XMLRPC::Lite',
+        version => '0.712',
         feature => ['xmlrpc'],
     },
     {
@@ -236,6 +299,13 @@ sub OPTIONAL_MODULES {
         version => 0,
         feature => ['jsonrpc'],
     },
+    {
+        package => 'JSON-XS',
+        module  => 'JSON::XS',
+        # 2.0 is the first version that will work with JSON::RPC.
+        version => '2.0',
+        feature => ['jsonrpc_faster'],
+    },
     {
         package => 'Test-Taint',
         module  => 'Test::Taint',
@@ -246,7 +316,7 @@ sub OPTIONAL_MODULES {
         # We need the 'utf8_mode' method of HTML::Parser, for HTML::Scrubber.
         package => 'HTML-Parser',
         module  => 'HTML::Parser',
-        version => '3.40',
+        version => (vers_cmp($perl_ver, '5.13.3') > -1) ? '3.67' : '3.40',
         feature => ['html_desc'],
     },
     {
@@ -255,26 +325,41 @@ sub OPTIONAL_MODULES {
         version => 0,
         feature => ['html_desc'],
     },
+    {
+        # we need version 2.21 of Encode for mime_name
+        package => 'Encode',
+        module  => 'Encode',
+        version => 2.21,
+        feature => ['detect_charset'],
+    },
+    {
+        package => 'Encode-Detect',
+        module  => 'Encode::Detect',
+        version => 0,
+        feature => ['detect_charset'],
+    },
 
     # Inbound Email
-    {
-        package => 'Email-MIME-Attachment-Stripper',
-        module  => 'Email::MIME::Attachment::Stripper',
-        version => 0,
-        feature => ['inbound_email'],
-    },
     {
         package => 'Email-Reply',
         module  => 'Email::Reply',
         version => 0,
         feature => ['inbound_email'],
     },
+    {
+        package => 'HTML-FormatText-WithLinks',
+        module  => 'HTML::FormatText::WithLinks',
+        # We need 0.13 to set the "bold" marker to "*".
+        version => '0.13',
+        feature => ['inbound_email'],
+    },
 
     # Mail Queueing
     {
         package => 'TheSchwartz',
         module  => 'TheSchwartz',
-        version => 0,
+        # 1.07 supports the prioritization of jobs.
+        version => 1.07,
         feature => ['jobqueue'],
     },
     {
@@ -283,6 +368,12 @@ sub OPTIONAL_MODULES {
         version => 0,
         feature => ['jobqueue'],
     },
+    {
+        package => 'File-Slurp',
+        module  => 'File::Slurp',
+        version => '9999.13',
+        feature => ['jobqueue'],
+    },
 
     # mod_perl
     {
@@ -291,6 +382,27 @@ sub OPTIONAL_MODULES {
         version => '1.999022',
         feature => ['mod_perl'],
     },
+    {
+        package => 'Apache-SizeLimit',
+        module  => 'Apache2::SizeLimit',
+        # 0.96 properly determines process size on Linux.
+        version => '0.96',
+        feature => ['mod_perl'],
+    },
+
+    # typesniffer
+    {
+        package => 'File-MimeInfo',
+        module  => 'File::MimeInfo::Magic',
+        version => '0',
+        feature => ['typesniffer'],
+    },
+    {
+        package => 'IO-stringy',
+        module  => 'IO::Scalar',
+        version => '0',
+        feature => ['typesniffer'],
+    },
     );
 
     my $extra_modules = _get_extension_requirements('OPTIONAL_MODULES');
@@ -348,6 +460,8 @@ sub check_requirements {
     print "\n", install_string('checking_optional'), "\n" if $output;
     my $missing_optional = _check_missing(OPTIONAL_MODULES, $output);
 
+    my $missing_apache = _missing_apache_modules(APACHE_MODULES, $output);
+
     # If we're running on Windows, reset the input line terminator so that
     # console input works properly - loading CGI tends to mess it up
     $/ = "\015\012" if ON_WINDOWS;
@@ -358,6 +472,7 @@ sub check_requirements {
         one_dbd  => $have_one_dbd,
         missing  => $missing,
         optional => $missing_optional,
+        apache   => $missing_apache,
         any_missing => !$pass || scalar(@$missing_optional),
     };
 }
@@ -376,15 +491,52 @@ sub _check_missing {
     return \@missing;
 }
 
-# Returns the build ID of ActivePerl. If several versions of
-# ActivePerl are installed, it won't be able to know which one
-# you are currently running. But that's our best guess.
-sub _get_activestate_build_id {
-    eval 'use Win32::TieRegistry';
-    return 0 if $@;
-    my $key = Win32::TieRegistry->new('LMachine\Software\ActiveState\ActivePerl')
-      or return 0;
-    return $key->GetValue("CurrentVersion");
+sub _missing_apache_modules {
+    my ($modules, $output) = @_;
+    my $apachectl = _get_apachectl();
+    return [] if !$apachectl;
+    my $command = "$apachectl -t -D DUMP_MODULES";
+    my $cmd_info = `$command 2>&1`;
+    # If apachectl returned a value greater than 0, then there was an
+    # error parsing Apache's configuration, and we can't check modules.
+    my $retval = $?;
+    if ($retval > 0) {
+        print STDERR install_string('apachectl_failed', 
+            { command => $command, root => ROOT_USER }), "\n";
+        return [];
+    }
+    my @missing;
+    foreach my $module (keys %$modules) {
+        my $ok = _check_apache_module($module, $modules->{$module}, 
+                                      $cmd_info, $output);
+        push(@missing, $module) if !$ok;
+    }
+    return \@missing;
+}
+
+sub _get_apachectl {
+    foreach my $bin_name (APACHE) {
+        my $bin = bin_loc($bin_name);
+        return $bin if $bin;
+    }
+    # Try again with a possibly different path.
+    foreach my $bin_name (APACHE) {
+        my $bin = bin_loc($bin_name, APACHE_PATH);
+        return $bin if $bin;
+    }
+    return undef;
+}
+
+sub _check_apache_module {
+    my ($module, $config_name, $mod_info, $output) = @_;
+    my $ok;
+    if ($mod_info =~ /^\s+\Q$config_name\E\b/m) {
+        $ok = 1;
+    }
+    if ($output) {
+        _checking_for({ package => $module, ok => $ok });
+    }
+    return $ok;
 }
 
 sub print_module_instructions {
@@ -423,27 +575,20 @@ sub print_module_instructions {
         }
     }
 
-    # We only print the PPM repository note if we have to.
-    if ((!$output && @{$check_results->{missing}})
-        || ($output && $check_results->{any_missing}))
-    {
-        if (ON_WINDOWS) {
-            my $perl_ver = sprintf('%vd', $^V);
-            
-            # URL when running Perl 5.8.x.
-            my $url_to_theory58S = 'http://theoryx5.uwinnipeg.ca/ppms';
-            # Packages for Perl 5.10 are not compatible with Perl 5.8.
-            if (vers_cmp($perl_ver, '5.10') > -1) {
-                $url_to_theory58S = 'http://cpan.uwinnipeg.ca/PPMPackages/10xx/';
-            }
-            print colored(install_string('ppm_repo_add', 
-                                 { theory_url => $url_to_theory58S }), 'red');
-            # ActivePerls older than revision 819 require an additional command.
-            if (_get_activestate_build_id() < 819) {
-                print install_string('ppm_repo_up');
-            }
-        }
+    if (my @missing = @{ $check_results->{apache} }) {
+        print install_string('modules_message_apache');
+        my $missing_string = join(', ', @missing);
+        my $size = TABLE_WIDTH - 7;
+        printf "*    \%-${size}s *\n", $missing_string;
+        my $spaces = TABLE_WIDTH - 2;
+        print "*", (' ' x $spaces), "*\n";
+    }
 
+    my $need_module_instructions =  
+        ( (!$output and @{$check_results->{missing}})
+          or ($output and $check_results->{any_missing}) ) ? 1 : 0;
+
+    if ($need_module_instructions or @{ $check_results->{apache} }) {
         # If any output was required, we want to close the "table"
         print "*" x TABLE_WIDTH . "\n";
     }
@@ -470,20 +615,21 @@ sub print_module_instructions {
     }
 
     if (my @missing = @{$check_results->{missing}}) {
-        print colored(install_string('commands_required'), 'red') . "\n";
+        print colored(install_string('commands_required'), COLOR_ERROR), "\n";
         foreach my $package (@missing) {
             my $command = install_command($package);
             print "    $command\n";
         }
     }
 
-    if ($output && $check_results->{any_missing} && !ON_WINDOWS
+    if ($output && $check_results->{any_missing} && !ON_ACTIVESTATE
         && !$check_results->{hide_all}) 
     {
         print install_string('install_all', { perl => $^X });
     }
     if (!$check_results->{pass}) {
-        print colored(install_string('installation_failed'), 'red') . "\n\n";
+        print colored(install_string('installation_failed'), COLOR_ERROR),
+              "\n\n";
     }
 }
 
@@ -499,16 +645,18 @@ sub _translate_feature {
 sub check_graphviz {
     my ($output) = @_;
 
-    return 1 if (Bugzilla->params->{'webdotbase'} =~ /^https?:/);
+    my $webdotbase = Bugzilla->params->{'webdotbase'};
+    return 1 if $webdotbase =~ /^https?:/;
 
-    printf("Checking for %15s %-9s ", "GraphViz", "(any)") if $output;
+    my $return;
+    $return = 1 if -x $webdotbase;
 
-    my $return = 0;
-    if(-x Bugzilla->params->{'webdotbase'}) {
-        print "ok: found\n" if $output;
-        $return = 1;
-    } else {
-        print "not a valid executable: " . Bugzilla->params->{'webdotbase'} . "\n";
+    if ($output) {
+        _checking_for({ package => 'GraphViz', ok => $return });
+    }
+
+    if (!$return) {
+        print install_string('bad_executable', { bin => $webdotbase }), "\n";
     }
 
     my $webdotdir = bz_locations()->{'webdotdir'};
@@ -517,8 +665,8 @@ sub check_graphviz {
         my $htaccess = new IO::File("$webdotdir/.htaccess", 'r') 
             || die "$webdotdir/.htaccess: " . $!;
         if (!grep(/png/, $htaccess->getlines)) {
-            print "Dependency graph images are not accessible.\n";
-            print "delete $webdotdir/.htaccess and re-run checksetup.pl to fix.\n";
+            print STDERR install_string('webdot_bad_htaccess',
+                                        { dir => $webdotdir }), "\n";
         }
         $htaccess->close;
     }
@@ -526,8 +674,8 @@ sub check_graphviz {
     return $return;
 }
 
-# This was originally clipped from the libnet Makefile.PL, adapted here to
-# use the below vers_cmp routine for accurate version checking.
+# This was originally clipped from the libnet Makefile.PL, adapted here for
+# accurate version checking.
 sub have_vers {
     my ($params, $output) = @_;
     my $module  = $params->{module};
@@ -539,34 +687,30 @@ sub have_vers {
     my $wanted  = $params->{version};
 
     eval "require $module;";
+    # Don't let loading a module change the output-encoding of STDOUT
+    # or STDERR. (CGI.pm tries to set "binmode" on these file handles when
+    # it's loaded, and other modules may do the same in the future.)
+    Bugzilla::Install::Util::set_output_encoding();
 
     # VERSION is provided by UNIVERSAL::, and can be called even if
-    # the module isn't loaded.
-    my $vnum = $module->VERSION || -1;
+    # the module isn't loaded. We eval'uate ->VERSION because it can die
+    # when the version is not valid (yes, this happens from time to time).
+    # In that case, we use an uglier method to get the version.
+    my $vnum = eval { $module->VERSION };
+    if ($@) {
+        no strict 'refs';
+        $vnum = ${"${module}::VERSION"};
 
-    # CGI's versioning scheme went 2.75, 2.751, 2.752, 2.753, 2.76
-    # That breaks the standard version tests, so we need to manually correct
-    # the version
-    if ($module eq 'CGI' && $vnum =~ /(2\.7\d)(\d+)/) {
-        $vnum = $1 . "." . $2;
-    }
-    # CPAN did a similar thing, where it has versions like 1.9304.
-    if ($module eq 'CPAN' and $vnum =~ /^(\d\.\d{2})\d{2}$/) {
-        $vnum = $1;
+        # If we come here, then the version is not a valid one.
+        # We try to sanitize it.
+        if ($vnum =~ /^((\d+)(\.\d+)*)/) {
+            $vnum = $1;
+        }
     }
+    $vnum ||= -1;
 
-    my $vstr;
-    if ($vnum eq "-1") { # string compare just in case it's non-numeric
-        $vstr = install_string('module_not_found');
-    }
-    elsif (vers_cmp($vnum,"0") > -1) {
-        $vstr = install_string('module_found', { ver => $vnum });
-    }
-    else {
-        $vstr = install_string('module_unknown_version');
-    }
-
-    my $vok = (vers_cmp($vnum,$wanted) > -1);
+    # Must do a string comparison as $vnum may be of the form 5.10.1.
+    my $vok = ($vnum ne '-1' && version->new($vnum) >= version->new($wanted)) ? 1 : 0;
     my $blacklisted;
     if ($vok && $params->{blacklist}) {
         $blacklisted = grep($vnum =~ /$_/, @{$params->{blacklist}});
@@ -574,24 +718,58 @@ sub have_vers {
     }
 
     if ($output) {
-        my $ok           = $vok ? install_string('module_ok') : '';
-        my $black_string = $blacklisted ? install_string('blacklisted') : '';
-        my $want_string  = $wanted ? "v$wanted" : install_string('any');
-
-        $ok = "$ok:" if $ok;
-        my $str = sprintf "%s %19s %-9s $ok $vstr $black_string\n",
-                    install_string('checking_for'), $package, "($want_string)";
-        print $vok ? $str : colored($str, 'red');
+        _checking_for({ 
+            package => $package, ok => $vok, wanted => $wanted,
+            found   => $vnum, blacklisted => $blacklisted
+        });
     }
     
     return $vok ? 1 : 0;
 }
 
+sub _checking_for {
+    my ($params) = @_;
+    my ($package, $ok, $wanted, $blacklisted, $found) = 
+        @$params{qw(package ok wanted blacklisted found)};
+
+    my $ok_string = $ok ? install_string('module_ok') : '';
+
+    # If we're actually checking versions (like for Perl modules), then
+    # we have some rather complex logic to determine what we want to 
+    # show. If we're not checking versions (like for GraphViz) we just
+    # show "ok" or "not found".
+    if (exists $params->{found}) {
+        my $found_string;
+        # We do a string compare in case it's non-numeric. We make sure
+        # it's not a version object as negative versions are forbidden.
+        if ($found && !ref($found) && $found eq '-1') {
+            $found_string = install_string('module_not_found');
+        }
+        elsif ($found) {
+            $found_string = install_string('module_found', { ver => $found });
+        }
+        else {
+            $found_string = install_string('module_unknown_version');
+        }
+        $ok_string = $ok ? "$ok_string: $found_string" : $found_string;
+    }
+    elsif (!$ok) {
+        $ok_string = install_string('module_not_found');
+    }
+
+    my $black_string = $blacklisted ? install_string('blacklisted') : '';
+    my $want_string  = $wanted ? "v$wanted" : install_string('any');
+
+    my $str = sprintf "%s %20s %-11s $ok_string $black_string\n",
+                install_string('checking_for'), $package, "($want_string)";
+    print $ok ? $str : colored($str, COLOR_ERROR);
+}
+
 sub install_command {
     my $module = shift;
     my ($command, $package);
 
-    if (ON_WINDOWS) {
+    if (ON_ACTIVESTATE) {
         $command = 'ppm install %s';
         $package = $module->{package};
     }
@@ -708,10 +886,12 @@ a hashref in the format of items from L</REQUIRED_MODULES>.
 
 =item C<optional> - The same as C<missing>, but for optional modules.
 
+=item C<apache> - The name of each optional Apache module that is missing.
+
 =item C<have_one_dbd> - True if at least one C<DBD::> module is installed.
 
-=item C<any_missing> - True if there are any missing modules, even optional
-modules.
+=item C<any_missing> - True if there are any missing Perl modules, even
+optional modules.
 
 =back
 
