@@ -1,20 +1,9 @@
-# -*- Mode: perl; indent-tabs-mode: nil -*-
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at http://mozilla.org/MPL/2.0/.
 #
-# The contents of this file are subject to the Mozilla Public
-# License Version 1.1 (the "License"); you may not use this file
-# except in compliance with the License. You may obtain a copy of
-# the License at http://www.mozilla.org/MPL/
-#
-# Software distributed under the License is distributed on an "AS
-# IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
-# implied. See the License for the specific language governing
-# rights and limitations under the License.
-#
-# The Original Code is the Bugzilla Bug Tracking System.
-#
-# Contributor(s): Tiago R. Mello <timello@async.com.br>
-#                 Max Kanat-Alexander <mkanat@bugzilla.org>
-#                 Frédéric Buclin <LpSolit@gmail.com>
+# This Source Code Form is "Incompatible With Secondary Licenses", as
+# defined by the Mozilla Public License, v. 2.0.
 
 use strict;
 
@@ -26,6 +15,8 @@ use Bugzilla::Constants;
 use Bugzilla::Util;
 use Bugzilla::Error;
 
+use Scalar::Util qw(blessed);
+
 ################################
 #####    Initialization    #####
 ################################
@@ -41,25 +32,28 @@ use constant DB_COLUMNS => qw(
     value
     product_id
     sortkey
+    isactive
 );
 
-use constant REQUIRED_CREATE_FIELDS => qw(
-    name
-    product
-);
+use constant REQUIRED_FIELD_MAP => {
+    product_id => 'product',
+};
 
 use constant UPDATE_COLUMNS => qw(
     value
     sortkey
+    isactive
 );
 
 use constant VALIDATORS => {
-    product => \&_check_product,
-    sortkey => \&_check_sortkey,
+    product  => \&_check_product,
+    sortkey  => \&_check_sortkey,
+    value    => \&_check_value,
+    isactive => \&Bugzilla::Object::check_boolean,
 };
 
-use constant UPDATE_VALIDATORS => {
-    value => \&_check_value,
+use constant VALIDATOR_DEPENDENCIES => {
+    value => ['product'],
 };
 
 ################################
@@ -70,7 +64,7 @@ sub new {
     my $dbh = Bugzilla->dbh;
 
     my $product;
-    if (ref $param) {
+    if (ref $param and !defined $param->{id}) {
         $product = $param->{product};
         my $name = $param->{name};
         if (!defined $product) {
@@ -94,23 +88,21 @@ sub new {
 }
 
 sub run_create_validators {
-    my $class  = shift;
+    my $class = shift;
     my $params = $class->SUPER::run_create_validators(@_);
-
     my $product = delete $params->{product};
     $params->{product_id} = $product->id;
-    $params->{value} = $class->_check_value($params->{name}, $product);
-    delete $params->{name};
-
     return $params;
 }
 
 sub update {
     my $self = shift;
+    my $dbh = Bugzilla->dbh;
+
+    $dbh->bz_start_transaction();
     my $changes = $self->SUPER::update(@_);
 
     if (exists $changes->{value}) {
-        my $dbh = Bugzilla->dbh;
         # The milestone value is stored in the bugs table instead of its ID.
         $dbh->do('UPDATE bugs SET target_milestone = ?
                   WHERE target_milestone = ? AND product_id = ?',
@@ -121,6 +113,8 @@ sub update {
                   WHERE id = ? AND defaultmilestone = ?',
                  undef, ($self->name, $self->product_id, $changes->{value}->[0]));
     }
+    $dbh->bz_commit_transaction();
+
     return $changes;
 }
 
@@ -128,6 +122,8 @@ sub remove_from_db {
     my $self = shift;
     my $dbh = Bugzilla->dbh;
 
+    $dbh->bz_start_transaction();
+
     # The default milestone cannot be deleted.
     if ($self->name eq $self->product->default_milestone) {
         ThrowUserError('milestone_is_default', { milestone => $self });
@@ -156,8 +152,9 @@ sub remove_from_db {
                              Bugzilla->user->id, $timestamp);
         }
     }
+    $self->SUPER::remove_from_db();
 
-    $dbh->do('DELETE FROM milestones WHERE id = ?', undef, $self->id);
+    $dbh->bz_commit_transaction();
 }
 
 ################################
@@ -165,7 +162,8 @@ sub remove_from_db {
 ################################
 
 sub _check_value {
-    my ($invocant, $name, $product) = @_;
+    my ($invocant, $name, undef, $params) = @_;
+    my $product = blessed($invocant) ? $invocant->product : $params->{product};
 
     $name = trim($name);
     $name || ThrowUserError('milestone_blank_name');
@@ -173,7 +171,6 @@ sub _check_value {
         ThrowUserError('milestone_name_too_long', {name => $name});
     }
 
-    $product = $invocant->product if (ref $invocant);
     my $milestone = new Bugzilla::Milestone({product => $product, name => $name});
     if ($milestone && (!ref $invocant || $milestone->id != $invocant->id)) {
         ThrowUserError('milestone_already_exists', { name    => $milestone->name,
@@ -196,6 +193,8 @@ sub _check_sortkey {
 
 sub _check_product {
     my ($invocant, $product) = @_;
+    $product || ThrowCodeError('param_required',
+                    { function => "$invocant->create", param => "product" });
     return Bugzilla->user->check_can_admin_product($product->name);
 }
 
@@ -203,8 +202,9 @@ sub _check_product {
 # Methods
 ################################
 
-sub set_name { $_[0]->set('value', $_[1]); }
-sub set_sortkey { $_[0]->set('sortkey', $_[1]); }
+sub set_name      { $_[0]->set('value', $_[1]);    }
+sub set_sortkey   { $_[0]->set('sortkey', $_[1]);  }
+sub set_is_active { $_[0]->set('isactive', $_[1]); }
 
 sub bug_count {
     my $self = shift;
@@ -226,6 +226,7 @@ sub bug_count {
 sub name       { return $_[0]->{'value'};      }
 sub product_id { return $_[0]->{'product_id'}; }
 sub sortkey    { return $_[0]->{'sortkey'};    }
+sub is_active  { return $_[0]->{'isactive'};   }
 
 sub product {
     my $self = shift;
@@ -247,7 +248,9 @@ Bugzilla::Milestone - Bugzilla product milestone class.
 
     use Bugzilla::Milestone;
 
-    my $milestone = new Bugzilla::Milestone({ name => $name, product => $product });
+    my $milestone = new Bugzilla::Milestone({ name => $name, product => $product_obj });
+    my $milestone = Bugzilla::Milestone->check({ name => $name, product => $product_obj });
+    my $milestone = Bugzilla::Milestone->check({ id => $id });
 
     my $name       = $milestone->name;
     my $product_id = $milestone->product_id;
@@ -255,7 +258,7 @@ Bugzilla::Milestone - Bugzilla product milestone class.
     my $sortkey    = $milestone->sortkey;
 
     my $milestone = Bugzilla::Milestone->create(
-        { name => $name, product => $product, sortkey => $sortkey });
+        { value => $name, product => $product, sortkey => $sortkey });
 
     $milestone->set_name($new_name);
     $milestone->set_sortkey($new_sortkey);
@@ -271,7 +274,7 @@ Milestone.pm represents a Product Milestone object.
 
 =over
 
-=item C<new({name => $name, product => $product})>
+=item C<< new({name => $name, product => $product}) >>
 
  Description: The constructor is used to load an existing milestone
               by passing a product object and a milestone name.
@@ -361,11 +364,11 @@ Milestone.pm represents a Product Milestone object.
 
 =over
 
-=item C<create({name => $name, product => $product, sortkey => $sortkey})>
+=item C<< create({value => $value, product => $product, sortkey => $sortkey}) >>
 
  Description: Create a new milestone for the given product.
 
- Params:      $name    - name of the new milestone (string). This name
+ Params:      $value   - name of the new milestone (string). This name
                          must be unique within the product.
               $product - a Bugzilla::Product object.
               $sortkey - the sortkey of the new milestone (signed integer)
