@@ -1,22 +1,9 @@
-# -*- Mode: perl; indent-tabs-mode: nil -*-
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at http://mozilla.org/MPL/2.0/.
 #
-# The contents of this file are subject to the Mozilla Public
-# License Version 1.1 (the "License"); you may not use this file
-# except in compliance with the License. You may obtain a copy of
-# the License at http://www.mozilla.org/MPL/
-#
-# Software distributed under the License is distributed on an "AS
-# IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
-# implied. See the License for the specific language governing
-# rights and limitations under the License.
-#
-# The Original Code is the Bugzilla Bug Tracking System.
-#
-# The Initial Developer of the Original Code is Everything Solved.
-# Portions created by Everything Solved are Copyright (C) 2006 
-# Everything Solved. All Rights Reserved.
-#
-# Contributor(s): Max Kanat-Alexander <mkanat@bugzilla.org>
+# This Source Code Form is "Incompatible With Secondary Licenses", as
+# defined by the Mozilla Public License, v. 2.0.
 
 use strict;
 
@@ -39,25 +26,25 @@ use Scalar::Util qw(blessed);
 #############
 
 use constant DB_TABLE => 'namedqueries';
+# Do not track buglists saved by users.
+use constant AUDIT_CREATES => 0;
+use constant AUDIT_UPDATES => 0;
+use constant AUDIT_REMOVES => 0;
 
 use constant DB_COLUMNS => qw(
     id
     userid
     name
     query
-    query_type
 );
 
-use constant REQUIRED_CREATE_FIELDS => qw(name query);
-
 use constant VALIDATORS => {
     name       => \&_check_name,
     query      => \&_check_query,
-    query_type => \&_check_query_type,
     link_in_footer => \&_check_link_in_footer,
 };
 
-use constant UPDATE_COLUMNS => qw(name query query_type);
+use constant UPDATE_COLUMNS => qw(name query);
 
 ###############
 # Constructor #
@@ -109,7 +96,7 @@ sub check {
     if (!$search->shared_with_group
         or !$user->in_group($search->shared_with_group)) 
     {
-        ThrowUserError('missing_query', { queryname => $search->name, 
+        ThrowUserError('missing_query', { name => $search->name,
                                           sharer_id => $search->user->id });
     }
 
@@ -143,12 +130,6 @@ sub _check_query {
     return $cgi->query_string;
 }
 
-sub _check_query_type {
-    my ($invocant, $type) = @_;
-    # Right now the only query type is LIST_OF_BUGS.
-    return $type ? LIST_OF_BUGS : QUERY_LIST;
-}
-
 #########################
 # Database Manipulation #
 #########################
@@ -176,6 +157,40 @@ sub create {
     return $obj;
 }
 
+sub rename_field_value {
+    my ($class, $field, $old_value, $new_value) = @_;
+
+    my $old = url_quote($old_value);
+    my $new = url_quote($new_value);
+    my $old_sql = $old;
+    $old_sql =~ s/([_\%])/\\$1/g;
+
+    my $table = $class->DB_TABLE;
+    my $id_field = $class->ID_FIELD;
+
+    my $dbh = Bugzilla->dbh;
+    $dbh->bz_start_transaction();
+
+    my %queries = @{ $dbh->selectcol_arrayref(
+        "SELECT $id_field, query FROM $table WHERE query LIKE ?",
+        {Columns=>[1,2]}, "\%$old_sql\%") };
+    foreach my $id (keys %queries) {
+        my $query = $queries{$id};
+        $query =~ s/\b$field=\Q$old\E\b/$field=$new/gi;
+        # Fix boolean charts.
+        while ($query =~ /\bfield(\d+-\d+-\d+)=\Q$field\E\b/gi) {
+            my $chart_id = $1;
+            # Note that this won't handle lists or substrings inside of
+            # boolean charts. Users will have to fix those themselves.
+            $query =~ s/\bvalue\Q$chart_id\E=\Q$old\E\b/value$chart_id=$new/i;
+        }
+        $dbh->do("UPDATE $table SET query = ? WHERE $id_field = ?",
+                 undef, $query, $id);
+    }
+
+    $dbh->bz_commit_transaction();
+}
+
 sub preload {
     my ($searches) = @_;
     my $dbh = Bugzilla->dbh;
@@ -269,7 +284,6 @@ sub shared_with_users {
 # Simple Accessors #
 ####################
 
-sub type { return $_[0]->{'query_type'}; }
 sub url  { return $_[0]->{'query'}; }
 
 sub user {
@@ -285,7 +299,6 @@ sub user {
 
 sub set_name       { $_[0]->set('name',       $_[1]); }
 sub set_url        { $_[0]->set('query',      $_[1]); }
-sub set_query_type { $_[0]->set('query_type', $_[1]); }
 
 1;
 
