@@ -1,27 +1,11 @@
-# -*- Mode: perl; indent-tabs-mode: nil -*-
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at http://mozilla.org/MPL/2.0/.
 #
-# The contents of this file are subject to the Mozilla Public
-# License Version 1.1 (the "License"); you may not use this file
-# except in compliance with the License. You may obtain a copy of
-# the License at http://www.mozilla.org/MPL/
-#
-# Software distributed under the License is distributed on an "AS
-# IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
-# implied. See the License for the specific language governing
-# rights and limitations under the License.
-#
-# The Original Code is the Bugzilla Bug Tracking System.
-#
-# The Initial Developer of the Original Code is Netscape Communications
-# Corporation. Portions created by Netscape are
-# Copyright (C) 1998 Netscape Communications Corporation. All
-# Rights Reserved.
-#
-# Contributor(s): Gervase Markham <gerv@gerv.net>
-#                 Lance Larsh <lance.larsh@oracle.com>
+# This Source Code Form is "Incompatible With Secondary Licenses", as
+# defined by the Mozilla Public License, v. 2.0.
 
 use strict;
-use lib ".";
 
 # This module implements a series - a set of data to be plotted on a chart.
 #
@@ -34,7 +18,11 @@ package Bugzilla::Series;
 
 use Bugzilla::Error;
 use Bugzilla::Util;
-use Bugzilla::User;
+
+# This is a hack so that we can re-use the rename_field_value
+# code from Bugzilla::Search::Saved.
+use constant DB_TABLE => 'series';
+use constant ID_FIELD => 'series_id';
 
 sub new {
     my $invocant = shift;
@@ -70,7 +58,8 @@ sub new {
     elsif ($arg_count >= 6 && $arg_count <= 8) {
         # We've been given a load of parameters to create a new Series from.
         # Currently, undef is always passed as the first parameter; this allows
-        # you to call writeToDatabase() unconditionally. 
+        # you to call writeToDatabase() unconditionally.
+        # XXX - You cannot set category_id and subcategory_id from here.
         $self->initFromParameters(@_);
     }
     else {
@@ -81,34 +70,30 @@ sub new {
 }
 
 sub initFromDatabase {
-    my $self = shift;
-    my $series_id = shift;
-    
+    my ($self, $series_id) = @_;
+    my $dbh = Bugzilla->dbh;
+    my $user = Bugzilla->user;
+
     detaint_natural($series_id) 
       || ThrowCodeError("invalid_series_id", { 'series_id' => $series_id });
-    
-    my $dbh = Bugzilla->dbh;
+
+    my $grouplist = $user->groups_as_string;
+
     my @series = $dbh->selectrow_array("SELECT series.series_id, cc1.name, " .
         "cc2.name, series.name, series.creator, series.frequency, " .
-        "series.query, series.is_public " .
+        "series.query, series.is_public, series.category, series.subcategory " .
         "FROM series " .
-        "LEFT JOIN series_categories AS cc1 " .
+        "INNER JOIN series_categories AS cc1 " .
         "    ON series.category = cc1.id " .
-        "LEFT JOIN series_categories AS cc2 " .
+        "INNER JOIN series_categories AS cc2 " .
         "    ON series.subcategory = cc2.id " .
         "LEFT JOIN category_group_map AS cgm " .
         "    ON series.category = cgm.category_id " .
-        "LEFT JOIN user_group_map AS ugm " .
-        "    ON cgm.group_id = ugm.group_id " .
-        "    AND ugm.user_id = " . Bugzilla->user->id .
-        "    AND isbless = 0 " .
-        "WHERE series.series_id = $series_id AND " .
-        "(is_public = 1 OR creator = " . Bugzilla->user->id . " OR " .
-        "(ugm.group_id IS NOT NULL)) " . 
-        $dbh->sql_group_by('series.series_id', 'cc1.name, cc2.name, ' .
-                           'series.name, series.creator, series.frequency, ' .
-                           'series.query, series.is_public'));
-    
+        "    AND cgm.group_id NOT IN($grouplist) " .
+        "WHERE series.series_id = ? " .
+        "    AND (creator = ? OR (is_public = 1 AND cgm.category_id IS NULL))",
+        undef, ($series_id, $user->id));
+
     if (@series) {
         $self->initFromParameters(@series);
         return $self;
@@ -123,8 +108,13 @@ sub initFromParameters {
     my $self = shift;
 
     ($self->{'series_id'}, $self->{'category'},  $self->{'subcategory'},
-     $self->{'name'}, $self->{'creator'}, $self->{'frequency'},
-     $self->{'query'}, $self->{'public'}) = @_;
+     $self->{'name'}, $self->{'creator_id'}, $self->{'frequency'},
+     $self->{'query'}, $self->{'public'}, $self->{'category_id'},
+     $self->{'subcategory_id'}) = @_;
+
+    # If the first parameter is undefined, check if this series already
+    # exists and update it series_id accordingly
+    $self->{'series_id'} ||= $self->existsInDatabase();
 }
 
 sub initFromCGI {
@@ -149,7 +139,7 @@ sub initFromCGI {
     $self->{'name'} = $cgi->param('name')
       || ThrowUserError("missing_name");
 
-    $self->{'creator'} = Bugzilla->user->id;
+    $self->{'creator_id'} = Bugzilla->user->id;
 
     $self->{'frequency'} = $cgi->param('frequency');
     detaint_natural($self->{'frequency'})
@@ -171,7 +161,7 @@ sub writeToDatabase {
     my $self = shift;
 
     my $dbh = Bugzilla->dbh;
-    $dbh->bz_lock_tables('series_categories WRITE', 'series WRITE');
+    $dbh->bz_start_transaction();
 
     my $category_id = getCategoryID($self->{'category'});
     my $subcategory_id = getCategoryID($self->{'subcategory'});
@@ -200,7 +190,7 @@ sub writeToDatabase {
         $dbh->do("INSERT INTO series (creator, category, subcategory, " .
                  "name, frequency, query, is_public) VALUES " . 
                  "(?, ?, ?, ?, ?, ?, ?)", undef,
-                 $self->{'creator'}, $category_id, $subcategory_id, $self->{'name'},
+                 $self->{'creator_id'}, $category_id, $subcategory_id, $self->{'name'},
                  $self->{'frequency'}, $self->{'query'}, $self->{'public'});
 
         # Retrieve series_id
@@ -210,7 +200,7 @@ sub writeToDatabase {
           || ThrowCodeError("missing_series_id", { 'series' => $self });
     }
     
-    $dbh->bz_unlock_tables();
+    $dbh->bz_commit_transaction();
 }
 
 # Check whether a series with this name, category and subcategory exists in
@@ -255,4 +245,27 @@ sub getCategoryID {
     return $category_id;
 }
 
+##########
+# Methods
+##########
+sub id   { return $_[0]->{'series_id'}; }
+sub name { return $_[0]->{'name'}; }
+
+sub creator {
+    my $self = shift;
+
+    if (!$self->{creator} && $self->{creator_id}) {
+        require Bugzilla::User;
+        $self->{creator} = new Bugzilla::User($self->{creator_id});
+    }
+    return $self->{creator};
+}
+
+sub remove_from_db {
+    my $self = shift;
+    my $dbh = Bugzilla->dbh;
+
+    $dbh->do('DELETE FROM series WHERE series_id = ?', undef, $self->id);
+}
+
 1;
