@@ -1,19 +1,9 @@
-# -*- Mode: perl; indent-tabs-mode: nil -*-
-#
-# The contents of this file are subject to the Mozilla Public
-# License Version 1.1 (the "License"); you may not use this file
-# except in compliance with the License. You may obtain a copy of
-# the License at http://www.mozilla.org/MPL/
-#
-# Software distributed under the License is distributed on an "AS
-# IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
-# implied. See the License for the specific language governing
-# rights and limitations under the License.
-#
-# The Original Code is the Bugzilla Bug Tracking System.
-#
-# Contributor(s): Marc Schumann <wurblzap@gmail.com>
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at http://mozilla.org/MPL/2.0/.
 #
+# This Source Code Form is "Incompatible With Secondary Licenses", as
+# defined by the Mozilla Public License, v. 2.0.
 
 
 package Bugzilla::User::Setting::Skin;
@@ -40,12 +30,8 @@ sub legal_values {
 
     foreach my $direntry (glob(catdir($dirbase, '*'))) {
         if (-d $direntry) {
-            # Stylesheet set
             next if basename($direntry) =~ /^cvs$/i;
-            push(@legal_values, basename($direntry));
-        }
-        elsif ($direntry =~ /\.css$/) {
-            # Single-file stylesheet
+            # Stylesheet set found
             push(@legal_values, basename($direntry));
         }
     }
