@@ -1,20 +1,9 @@
-# -*- Mode: perl; indent-tabs-mode: nil -*-
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at http://mozilla.org/MPL/2.0/.
 #
-# The contents of this file are subject to the Mozilla Public
-# License Version 1.1 (the "License"); you may not use this file
-# except in compliance with the License. You may obtain a copy of
-# the License at http://www.mozilla.org/MPL/
-#
-# Software distributed under the License is distributed on an "AS
-# IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
-# implied. See the License for the specific language governing
-# rights and limitations under the License.
-#
-# The Original Code is the Bugzilla Bug Tracking System.
-#
-# Contributor(s): Tiago R. Mello <timello@async.com.br>
-#                 Max Kanat-Alexander <mkanat@bugzilla.org>
-#                 Frédéric Buclin <LpSolit@gmail.com>
+# This Source Code Form is "Incompatible With Secondary Licenses", as
+# defined by the Mozilla Public License, v. 2.0.
 
 use strict;
 
@@ -26,6 +15,8 @@ use Bugzilla::Install::Util qw(vers_cmp);
 use Bugzilla::Util;
 use Bugzilla::Error;
 
+use Scalar::Util qw(blessed);
+
 ################################
 #####   Initialization     #####
 ################################
@@ -42,23 +33,26 @@ use constant DB_COLUMNS => qw(
     id
     value
     product_id
+    isactive
 );
 
-use constant REQUIRED_CREATE_FIELDS => qw(
-    name
-    product
-);
+use constant REQUIRED_FIELD_MAP => {
+    product_id => 'product',
+};
 
 use constant UPDATE_COLUMNS => qw(
     value
+    isactive
 );
 
 use constant VALIDATORS => {
-    product => \&_check_product,
+    product  => \&_check_product,
+    value    => \&_check_value,
+    isactive => \&Bugzilla::Object::check_boolean,
 };
 
-use constant UPDATE_VALIDATORS => {
-    value => \&_check_value,
+use constant VALIDATOR_DEPENDENCIES => {
+    value => ['product'],
 };
 
 ################################
@@ -71,7 +65,7 @@ sub new {
     my $dbh = Bugzilla->dbh;
 
     my $product;
-    if (ref $param) {
+    if (ref $param and !defined $param->{id}) {
         $product = $param->{product};
         my $name = $param->{name};
         if (!defined $product) {
@@ -103,12 +97,8 @@ sub new_from_list {
 sub run_create_validators {
     my $class  = shift;
     my $params = $class->SUPER::run_create_validators(@_);
-
     my $product = delete $params->{product};
     $params->{product_id} = $product->id;
-    $params->{value} = $class->_check_value($params->{name}, $product);
-    delete $params->{name};
-
     return $params;
 }
 
@@ -127,14 +117,18 @@ sub bug_count {
 
 sub update {
     my $self = shift;
+    my $dbh = Bugzilla->dbh;
+
+    $dbh->bz_start_transaction();
     my ($changes, $old_self) = $self->SUPER::update(@_);
 
     if (exists $changes->{value}) {
-        my $dbh = Bugzilla->dbh;
         $dbh->do('UPDATE bugs SET version = ?
                   WHERE version = ? AND product_id = ?',
                   undef, ($self->name, $old_self->name, $self->product_id));
     }
+    $dbh->bz_commit_transaction();
+
     return $changes;
 }
 
@@ -142,12 +136,21 @@ sub remove_from_db {
     my $self = shift;
     my $dbh = Bugzilla->dbh;
 
+    $dbh->bz_start_transaction();
+
+    # Products must have at least one version.
+    if (scalar(@{$self->product->versions}) == 1) {
+        ThrowUserError('version_is_last', { version => $self });
+    }
+
     # The version cannot be removed if there are bugs
     # associated with it.
     if ($self->bug_count) {
         ThrowUserError("version_has_bugs", { nb => $self->bug_count });
     }
     $self->SUPER::remove_from_db();
+
+    $dbh->bz_commit_transaction();
 }
 
 ###############################
@@ -155,6 +158,7 @@ sub remove_from_db {
 ###############################
 
 sub product_id { return $_[0]->{'product_id'}; }
+sub is_active  { return $_[0]->{'isactive'};   }
 
 sub product {
     my $self = shift;
@@ -168,17 +172,18 @@ sub product {
 # Validators
 ################################
 
-sub set_name { $_[0]->set('value', $_[1]); }
+sub set_name      { $_[0]->set('value', $_[1]);    }
+sub set_is_active { $_[0]->set('isactive', $_[1]); }
 
 sub _check_value {
-    my ($invocant, $name, $product) = @_;
+    my ($invocant, $name, undef, $params) = @_;
+    my $product = blessed($invocant) ? $invocant->product : $params->{product};
 
     $name = trim($name);
     $name || ThrowUserError('version_blank_name');
     # Remove unprintable characters
     $name = clean_text($name);
 
-    $product = $invocant->product if (ref $invocant);
     my $version = new Bugzilla::Version({ product => $product, name => $name });
     if ($version && (!ref $invocant || $version->id != $invocant->id)) {
         ThrowUserError('version_already_exists', { name    => $version->name,
@@ -189,6 +194,8 @@ sub _check_value {
 
 sub _check_product {
     my ($invocant, $product) = @_;
+    $product || ThrowCodeError('param_required',
+                    { function => "$invocant->create", param => 'product' });
     return Bugzilla->user->check_can_admin_product($product->name);
 }
 
@@ -204,14 +211,16 @@ Bugzilla::Version - Bugzilla product version class.
 
     use Bugzilla::Version;
 
-    my $version = new Bugzilla::Version({ name => $name, product => $product });
+    my $version = new Bugzilla::Version({ name => $name, product => $product_obj });
+    my $version = Bugzilla::Version->check({ name => $name, product => $product_obj });
+    my $version = Bugzilla::Version->check({ id => $id });
 
     my $value = $version->name;
     my $product_id = $version->product_id;
     my $product = $version->product;
 
     my $version = Bugzilla::Version->create(
-        { name => $name, product => $product });
+        { value => $name, product => $product_obj });
 
     $version->set_name($new_name);
     $version->update();
