@@ -1,40 +1,10 @@
 #!/usr/bin/perl -w
-# -*- Mode: perl; indent-tabs-mode: nil -*-
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at http://mozilla.org/MPL/2.0/.
 #
-# The contents of this file are subject to the Mozilla Public
-# License Version 1.1 (the "License"); you may not use this file
-# except in compliance with the License. You may obtain a copy of
-# the License at http://www.mozilla.org/MPL/
-#
-# Software distributed under the License is distributed on an "AS
-# IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
-# implied. See the License for the specific language governing
-# rights and limitations under the License.
-#
-# The Original Code is mozilla.org code.
-#
-# The Initial Developer of the Original Code is Holger
-# Schurig. Portions created by Holger Schurig are
-# Copyright (C) 1999 Holger Schurig. All
-# Rights Reserved.
-#
-# Contributor(s): Holger Schurig <holgerschurig@nikocity.de>
-#                 Terry Weissman <terry@mozilla.org>
-#                 Dan Mosedale <dmose@mozilla.org>
-#                 Dave Miller <justdave@syndicomm.com>
-#                 Zach Lipton  <zach@zachlipton.com>
-#                 Jacob Steenhagen <jake@bugzilla.org>
-#                 Bradley Baetz <bbaetz@student.usyd.edu.au>
-#                 Tobias Burnus <burnus@net-b.de>
-#                 Shane H. W. Travis <travis@sedsystems.ca>
-#                 Gervase Markham <gerv@gerv.net>
-#                 Erik Stambaugh <erik@dasbistro.com>
-#                 Dave Lawrence <dkl@redhat.com>
-#                 Max Kanat-Alexander <mkanat@bugzilla.org>
-#                 Joel Peshkin <bugreport@peshkin.net>
-#                 Lance Larsh <lance.larsh@oracle.com>
-#                 A. Karl Kornel <karl@kornel.name>
-#                 Marc Schumann <wurblzap@gmail.com>
+# This Source Code Form is "Incompatible With Secondary Licenses", as
+# defined by the Mozilla Public License, v. 2.0.
 
 # This file has detailed POD docs, do "perldoc checksetup.pl" to see them.
 
@@ -53,21 +23,19 @@ BEGIN { chdir dirname($0); }
 use lib qw(. lib);
 use Bugzilla::Constants;
 use Bugzilla::Install::Requirements;
-use Bugzilla::Install::Util qw(install_string get_version_and_os init_console);
+use Bugzilla::Install::Util qw(install_string get_version_and_os 
+                               init_console success);
 
 ######################################################################
 # Live Code
 ######################################################################
 
+# Do not run checksetup.pl from the web browser.
+Bugzilla::Install::Util::no_checksetup_from_cgi() if $ENV{'SERVER_SOFTWARE'};
+
 # When we're running at the command line, we need to pick the right
 # language before ever displaying any string.
 init_console();
-# Required for displaying strings from install_string, which are always
-# in UTF-8, in every language. For other scripts, Bugzilla::init_page
-# handles this, but here we just need to assume that checksetup.pl output
-# is always UTF-8 in order for install_string to work properly in other
-# languages.
-binmode STDOUT, ':utf8';
 
 my %switch;
 GetOptions(\%switch, 'help|h|?', 'check-modules', 'no-templates|t',
@@ -104,6 +72,9 @@ exit if $switch{'check-modules'};
 require Bugzilla;
 require Bugzilla::User;
 
+require Bugzilla::Util;
+import Bugzilla::Util qw(get_text);
+
 require Bugzilla::Config;
 import Bugzilla::Config qw(:admin);
 
@@ -196,6 +167,7 @@ Bugzilla::Field::populate_field_definitions();
 ###########################################################################
 
 Bugzilla::Install::DB::update_table_definitions(\%old_params);
+Bugzilla::Install::init_workflow();
 
 ###########################################################################
 # Bugzilla uses --GROUPS-- to assign various rights to its users.
@@ -237,9 +209,12 @@ Bugzilla::Hook::process('install_before_final_checks', { silent => $silent });
 # Check if the default parameter for urlbase is still set, and if so, give
 # notification that they should go and visit editparams.cgi 
 if (Bugzilla->params->{'urlbase'} eq '') {
-    print "\n" . Bugzilla::Install::get_text('install_urlbase_default') . "\n"
+    print "\n" . get_text('install_urlbase_default') . "\n"
         unless $silent;
 }
+if (!$silent) {
+    success(get_text('install_success'));
+}
 
 __END__
 
@@ -518,4 +493,3 @@ L<Bugzilla::Config/update_params>
 L<Bugzilla::DB/CONNECTION>
 
 =back
-
