@@ -1,38 +1,16 @@
 #!/usr/bin/perl -w
-# -*- Mode: perl; indent-tabs-mode: nil -*-
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at http://mozilla.org/MPL/2.0/.
 #
-# The contents of this file are subject to the Mozilla Public
-# License Version 1.1 (the "License"); you may not use this file
-# except in compliance with the License. You may obtain a copy of
-# the License at http://www.mozilla.org/MPL/
-#
-# Software distributed under the License is distributed on an "AS
-# IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
-# implied. See the License for the specific language governing
-# rights and limitations under the License.
-#
-# The Original Code is the Bugzilla Bug Tracking System.
-#
-# The Initial Developer of the Original Code is Netscape Communications
-# Corporation. Portions created by Netscape are
-# Copyright (C) 1998 Netscape Communications Corporation. All
-# Rights Reserved.
-#
-# Contributor(s): Terry Weissman <terry@mozilla.org>,
-#                 Harrison Page <harrison@netscape.com>
-#                 Gervase Markham <gerv@gerv.net>
-#                 Richard Walters <rwalters@qualcomm.com>
-#                 Jean-Sebastien Guay <jean_seb@hybride.com>
-#                 Frédéric Buclin <LpSolit@gmail.com>
-
-# Run me out of cron at midnight to collect Bugzilla statistics.
-#
-# To run new charts for a specific date, pass it in on the command line in
-# ISO (2004-08-14) format.
+# This Source Code Form is "Incompatible With Secondary Licenses", as
+# defined by the Mozilla Public License, v. 2.0.
 
 use strict;
 use lib qw(. lib);
 
+use Getopt::Long qw(:config bundling);
+use Pod::Usage;
 use List::Util qw(first);
 use Cwd;
 
@@ -44,6 +22,13 @@ use Bugzilla::Search;
 use Bugzilla::User;
 use Bugzilla::Product;
 use Bugzilla::Field;
+use Bugzilla::Install::Filesystem qw(fix_dir_permissions);
+
+my %switch;
+GetOptions(\%switch, 'help|h', 'regenerate');
+
+# Print the help message if that switch was selected.
+pod2usage({-verbose => 1, -exitval => 1}) if $switch{'help'};
 
 # Turn off output buffering (probably needed when displaying output feedback
 # in the regenerate mode).
@@ -63,17 +48,6 @@ if (chdir($graphsdir)) {
 
 my $dbh = Bugzilla->switch_to_shadow_db();
 
-
-# To recreate the daily statistics,  run "collectstats.pl --regenerate" .
-my $regenerate = 0;
-if ($#ARGV >= 0 && $ARGV[0] eq "--regenerate") {
-    shift(@ARGV);
-    $regenerate = 1;
-}
-
-my @myproducts = map {$_->name} Bugzilla::Product->get_all;
-unshift(@myproducts, "-All-");
-
 # As we can now customize statuses and resolutions, looking at the current list
 # of legal values only is not enough as some now removed statuses and resolutions
 # may have existed in the past, or have been renamed. We want them all.
@@ -115,7 +89,7 @@ my @resolutions = @{$fields->{'resolution'}};
 # per bug, per day. Instead, we now just get all the data out of the DB
 # at once and stuff it into some data structures.
 my (%bug_status, %bug_resolution, %removed);
-if ($regenerate) {
+if ($switch{'regenerate'}) {
     %bug_resolution = @{ $dbh->selectcol_arrayref(
         'SELECT bug_id, resolution FROM bugs', {Columns=>[1,2]}) };
     %bug_status = @{ $dbh->selectcol_arrayref(
@@ -143,42 +117,42 @@ if ($regenerate) {
 }
 
 my $tstart = time;
+
+my @myproducts = Bugzilla::Product->get_all;
+unshift(@myproducts, "-All-");
+
+my $dir = "$datadir/mining";
+if (!-d $dir) {
+    mkdir $dir or die "mkdir $dir failed: $!";
+    fix_dir_permissions($dir);
+}
+
 foreach (@myproducts) {
-    my $dir = "$datadir/mining";
-
-    &check_data_dir ($dir);
-
-    if ($regenerate) {
+    if ($switch{'regenerate'}) {
         regenerate_stats($dir, $_, \%bug_resolution, \%bug_status, \%removed);
     } else {
         &collect_stats($dir, $_);
     }
 }
+# Fix permissions for all files in mining/.
+fix_dir_permissions($dir);
+
 my $tend = time;
 # Uncomment the following line for performance testing.
-#print "Total time taken " . delta_time($tstart, $tend) . "\n";
+#say "Total time taken " . delta_time($tstart, $tend);
 
 CollectSeriesData();
 
-sub check_data_dir {
-    my $dir = shift;
-
-    if (! -d $dir) {
-        mkdir $dir, 0755;
-        chmod 0755, $dir;
-    }
-}
-
 sub collect_stats {
     my $dir = shift;
     my $product = shift;
     my $when = localtime (time);
     my $dbh = Bugzilla->dbh;
-
     my $product_id;
-    if ($product ne '-All-') {
-        my $prod = Bugzilla::Product::check_product($product);
-        $product_id = $prod->id;
+
+    if (ref $product) {
+        $product_id = $product->id;
+        $product = $product->name;
     }
 
     # NB: Need to mangle the product for the filename, but use the real
@@ -254,7 +228,6 @@ FIN
     }
     print DATA (join '|', @row) . "\n";
     close DATA;
-    chmod 0644, $file;
 }
 
 sub get_old_data {
@@ -313,6 +286,9 @@ sub regenerate_stats {
 
     # NB: Need to mangle the product for the filename, but use the real
     # product name in the query
+    if (ref $product) {
+        $product = $product->name;
+    }
     my $file_product = $product;
     $file_product =~ s/\//-/gs;
     my $file = join '/', $dir, $file_product;
@@ -407,14 +383,12 @@ FIN
             foreach (@resolutions) { print DATA "|$bugcount{$_}"; }
             print DATA "\n";
         }
-        
+
         # Finish up output feedback for this product.
         my $tend = time;
-        print "\rRegenerating $product \[100.0\%] - " .
-            delta_time($tstart, $tend) . "\n";
-            
+        say "\rRegenerating $product \[100.0\%] - " . delta_time($tstart, $tend);
+
         close DATA;
-        chmod 0640, $file;
     }
 }
 
@@ -472,7 +446,7 @@ sub CollectSeriesData {
     # (days_since_epoch + series_id) % frequency = 0. So they'll run every
     # <frequency> days, but the start date depends on the series_id.
     my $days_since_epoch = int(time() / (60 * 60 * 24));
-    my $today = $ARGV[0] || today_dash();
+    my $today = today_dash();
 
     # We save a copy of the main $dbh and then switch to the shadow and get
     # that one too. Remember, these may be the same.
@@ -506,11 +480,11 @@ sub CollectSeriesData {
         # Do not die if Search->new() detects invalid data, such as an obsolete
         # login name or a renamed product or component, etc.
         eval {
-            my $search = new Bugzilla::Search('params' => $cgi,
+            my $search = new Bugzilla::Search('params' => scalar $cgi->Vars,
                                               'fields' => ["bug_id"],
+                                              'allow_unlimited' => 1,
                                               'user'   => $user);
-            my $sql = $search->getSQL();
-            $data = $shadow_dbh->selectall_arrayref($sql);
+            $data = $search->data;
         };
 
         if (!$@) {
@@ -524,3 +498,39 @@ sub CollectSeriesData {
     }
 }
 
+__END__
+
+=head1 NAME
+
+collectstats.pl - Collect data about Bugzilla bugs.
+
+=head1 SYNOPSIS
+
+ ./collectstats.pl [--regenerate] [--help]
+
+Collects data about bugs to be used in Old and New Charts.
+
+=head1 OPTIONS
+
+=over
+
+=item B<--help>
+
+Print this help page.
+
+=item B<--regenerate>
+
+Recreate all the data about bugs, from day 1. This option is only relevant
+for Old Charts, and has no effect for New Charts.
+This option will overwrite all existing collected data and can take a huge
+amount of time. You normally don't need to use this option (do not use it
+in a cron job).
+
+=back
+
+=head1 DESCRIPTION
+
+This script collects data about all bugs for Old Charts, triaged by product
+and by bug status and resolution. It also collects data for New Charts, based
+on existing series. For New Charts, data is only collected once a series is
+defined; this script cannot recreate data prior to this date.
