@@ -1,22 +1,10 @@
 #!/usr/bin/perl -w
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at http://mozilla.org/MPL/2.0/.
 #
-# The contents of this file are subject to the Mozilla Public
-# License Version 1.1 (the "License"); you may not use this file
-# except in compliance with the License. You may obtain a copy of
-# the License at http://www.mozilla.org/MPL/
-#
-# Software distributed under the License is distributed on an "AS
-# IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
-# implied. See the License for the specific language governing
-# rights and limitations under the License.
-#
-# The Original Code is the Bugzilla Bug Tracking System.
-#
-# The Initial Developer of the Original Code is Everything Solved.
-# Portions created by Everything Solved are Copyright (C) 2006 
-# Everything Solved. All Rights Reserved.
-#
-# Contributor(s): Max Kanat-Alexander <mkanat@bugzilla.org>
+# This Source Code Form is "Incompatible With Secondary Licenses", as
+# defined by the Mozilla Public License, v. 2.0.
 
 use strict;
 use lib qw(. lib);
@@ -50,11 +38,16 @@ use constant TARGET_DB_HOST => 'localhost';
 
 print "Connecting to the '" . SOURCE_DB_NAME . "' source database on " 
       . SOURCE_DB_TYPE . "...\n";
-my $source_db = Bugzilla::DB::_connect(SOURCE_DB_TYPE, SOURCE_DB_HOST, 
-    SOURCE_DB_NAME, undef, undef, SOURCE_DB_USER, SOURCE_DB_PASSWORD);
+my $source_db = Bugzilla::DB::_connect({
+    db_driver => SOURCE_DB_TYPE, 
+    db_host   => SOURCE_DB_HOST, 
+    db_name   => SOURCE_DB_NAME, 
+    db_user   => SOURCE_DB_USER, 
+    db_pass   => SOURCE_DB_PASSWORD,
+});
 # Don't read entire tables into memory.
 if (SOURCE_DB_TYPE eq 'Mysql') {
-    $source_db->{'mysql_use_result'}=1;
+    $source_db->{'mysql_use_result'} = 1;
 
     # MySQL cannot have two queries running at the same time. Ensure the schema
     # is loaded from the database so bz_column_info will not execute a query
@@ -63,19 +56,22 @@ if (SOURCE_DB_TYPE eq 'Mysql') {
 
 print "Connecting to the '" . TARGET_DB_NAME . "' target database on "
       . TARGET_DB_TYPE . "...\n";
-my $target_db = Bugzilla::DB::_connect(TARGET_DB_TYPE, TARGET_DB_HOST, 
-    TARGET_DB_NAME, undef, undef, TARGET_DB_USER, TARGET_DB_PASSWORD);
+my $target_db = Bugzilla::DB::_connect({
+    db_driver => TARGET_DB_TYPE,
+    db_host   => TARGET_DB_HOST, 
+    db_name   => TARGET_DB_NAME, 
+    db_user   => TARGET_DB_USER,
+    db_pass   => TARGET_DB_PASSWORD,
+});
 my $ident_char = $target_db->get_info( 29 ); # SQL_IDENTIFIER_QUOTE_CHAR
 
 # We use the table list from the target DB, because if somebody
 # has customized their source DB, we still want the script to work,
 # and it may otherwise fail in that situation (that is, the tables
 # may not exist in the target DB).
-my @table_list = $target_db->bz_table_list_real();
-
-# We don't want to copy over the bz_schema table's contents.
-my $bz_schema_location = lsearch(\@table_list, 'bz_schema');
-splice(@table_list, $bz_schema_location, 1) if $bz_schema_location > 0;
+#
+# We don't want to copy over the bz_schema table's contents, though.
+my @table_list = grep { $_ ne 'bz_schema' } $target_db->bz_table_list_real();
 
 # Instead of figuring out some fancy algorithm to insert data in the right
 # order and not break FK integrity, we just drop them all.
@@ -253,4 +249,3 @@ in the "User-Configurable Settings" section at the top of the script.
 The C<SOURCE> settings are for the database you're copying from, and 
 the C<TARGET> settings are for the database you're copying to. The 
 C<DB_TYPE> is the name of a DB driver from the F<Bugzilla/DB/> directory.
-
