@@ -1,24 +1,10 @@
 #!/usr/bin/perl -w
-# -*- Mode: perl; indent-tabs-mode: nil -*-
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at http://mozilla.org/MPL/2.0/.
 #
-# The contents of this file are subject to the Mozilla Public License Version
-# 1.1 (the "License"); you may not use this file except in compliance with
-# the License. You may obtain a copy of the License at
-# http://www.mozilla.org/MPL/
-#
-# Software distributed under the License is distributed on an "AS IS" basis,
-# WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
-# for the specific language governing rights and limitations under the
-# License.
-#
-# The Original Code is the Bugzilla Bug Tracking System.
-#
-# The Initial Developer of the Original Code is the National Aeronautics
-# and Space Administration of the United States Government.
-# Portions created by the Initial Developer are Copyright (C) 2010
-# the Initial Developer. All Rights Reserved.
-#
-# Contributor(s): Jesse Clark <jjclark1982@gmail.com>
+# This Source Code Form is "Incompatible With Secondary Licenses", as
+# defined by the Mozilla Public License, v. 2.0.
 
 use File::Basename;
 BEGIN { chdir dirname($0) . "/.."; }
@@ -96,7 +82,7 @@ sub get_object {
     elsif (m/^\d+$/) {
         @results = ($class->new($_));
     }
-    elsif (m/\w/i && grep {$_ eq 'name'} ($class->DB_COLUMNS)) {
+    elsif (m/\w/i && grep {$_ eq 'name'} ($class->_get_db_columns)) {
         @results = @{$class->match({name => $_})};
     }
     else {
