@@ -1,23 +1,10 @@
 #!/usr/bin/perl -w
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at http://mozilla.org/MPL/2.0/.
 #
-# The contents of this file are subject to the Mozilla Public
-# License Version 1.1 (the "License"); you may not use this file
-# except in compliance with the License. You may obtain a copy of
-# the License at http://www.mozilla.org/MPL/
-#
-# Software distributed under the License is distributed on an "AS
-# IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
-# implied. See the License for the specific language governing
-# rights and limitations under the License.
-#
-# The Original Code is the Bugzilla Bug Tracking System.
-#
-# The Initial Developer of the Original Code is Everything Solved, Inc.
-# Portions created by the Initial Developer are Copyright (C) 2009 the
-# Initial Developer. All Rights Reserved.
-#
-# Contributor(s):
-#   Max Kanat-Alexander <mkanat@bugzilla.org>
+# This Source Code Form is "Incompatible With Secondary Licenses", as
+# defined by the Mozilla Public License, v. 2.0.
 
 use strict;
 use lib qw(. lib);
@@ -27,11 +14,11 @@ use Bugzilla::Error;
 use Bugzilla::Util qw(get_text);
 
 use File::Path qw(mkpath);
-use DateTime;
 
 my $base_dir = bz_locations()->{'extensionsdir'};
 
 my $name = $ARGV[0] or ThrowUserError('extension_create_no_name');
+$name = ucfirst($name);
 if ($name !~ /^[A-Z]/) {
     ThrowUserError('extension_first_letter_caps', { name => $name });
 }
@@ -41,20 +28,19 @@ mkpath($extension_dir)
   || die "$extension_dir already exists or cannot be created.\n";
 
 my $lcname = lc($name);
-foreach my $path (qw(lib template/en/default/hook), 
+foreach my $path (qw(lib web template/en/default/hook), 
                   "template/en/default/$lcname")
 {
     mkpath("$extension_dir/$path") || die "$extension_dir/$path: $!";
 }
 
-my $year = DateTime->now()->year;
-
 my $template = Bugzilla->template;
-my $vars = { year => $year, name => $name, path => $extension_dir };
+my $vars = { name => $name, path => $extension_dir };
 my %create_files = (
     'config.pm.tmpl'       => 'Config.pm',
     'extension.pm.tmpl'    => 'Extension.pm',
     'util.pm.tmpl'         => 'lib/Util.pm',
+    'web-readme.txt.tmpl'  => 'web/README',
     'hook-readme.txt.tmpl' => 'template/en/default/hook/README',
     'name-readme.txt.tmpl' => "template/en/default/$lcname/README",
 );
