@@ -1,29 +1,10 @@
 #!/usr/bin/perl -wT
-# -*- Mode: perl; indent-tabs-mode: nil -*-
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at http://mozilla.org/MPL/2.0/.
 #
-# The contents of this file are subject to the Mozilla Public
-# License Version 1.1 (the "License"); you may not use this file
-# except in compliance with the License. You may obtain a copy of
-# the License at http://www.mozilla.org/MPL/
-#
-# Software distributed under the License is distributed on an "AS
-# IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
-# implied. See the License for the specific language governing
-# rights and limitations under the License.
-#
-# The Original Code is the Bugzilla Bug Tracking System.
-#
-# The Initial Developer of the Original Code is Netscape Communications
-# Corporation. Portions created by Netscape are
-# Copyright (C) 1998 Netscape Communications Corporation. All
-# Rights Reserved.
-#
-# Contributor(s): Dawn Endico <endico@mozilla.org>
-#                 Gregary Hendricks <ghendricks@novell.com>
-#                 Vance Baarda <vrb@novell.com>
-#                 Guzman Braso <gbn@hqso.net>
-#                 Erik Purins <epurins@day1studios.com>
-#                 Frédéric Buclin <LpSolit@gmail.com>
+# This Source Code Form is "Incompatible With Secondary Licenses", as
+# defined by the Mozilla Public License, v. 2.0.
 
 # This script reads in xml bug data from standard input and inserts
 # a new bug into bugzilla. Everything before the beginning <?xml line
@@ -70,7 +51,9 @@ use lib qw(. lib);
 
 
 use Bugzilla;
+use Bugzilla::Object;
 use Bugzilla::Bug;
+use Bugzilla::Attachment;
 use Bugzilla::Product;
 use Bugzilla::Version;
 use Bugzilla::Component;
@@ -95,12 +78,18 @@ my $debug = 0;
 my $mail  = '';
 my $attach_path = '';
 my $help  = 0;
+my $bug_page = 'show_bug.cgi?id=';
+my $default_product_name = '';
+my $default_component_name = '';
 
 my $result = GetOptions(
     "verbose|debug+" => \$debug,
     "mail|sendmail!" => \$mail,
     "attach_path=s"  => \$attach_path,
-    "help|?"         => \$help
+    "help|?"         => \$help,
+    "bug_page=s"     => \$bug_page,
+    "product=s"      => \$default_product_name,
+    "component=s"    => \$default_component_name,
 );
 
 pod2usage(0) if $help;
@@ -126,7 +115,7 @@ sub MailMessage {
     my $subject    = shift;
     my $message    = shift;
     my @recipients = @_;
-    my $from   = $params->{"moved-from-address"};
+    my $from   = $params->{"mailfrom"};
     $from =~ s/@/\@/g;
 
     foreach my $to (@recipients){
@@ -166,7 +155,7 @@ sub flag_handler {
     my (
         $name,            $status,      $setter_login,
         $requestee_login, $exporterid,  $bugid,
-        $productid,       $componentid, $attachid
+        $componentid,     $productid,   $attachid
       )
       = @_;
 
@@ -316,22 +305,8 @@ sub init() {
     }
     Error( "no maintainer", "REOPEN", $exporter ) unless ($maintainer);
     Error( "no exporter",   "REOPEN", $exporter ) unless ($exporter);
-    Error( "bug importing is disabled here", undef, $exporter ) unless ( $params->{"move-enabled"} );
     Error( "invalid exporter: $exporter", "REOPEN", $exporter ) if ( !login_to_id($exporter) );
     Error( "no urlbase set", "REOPEN", $exporter ) unless ($urlbase);
-    my $def_product =
-        new Bugzilla::Product( { name => $params->{"moved-default-product"} } )
-        || Error("an invalid default product was defined for the target DB. " .
-                  $params->{"maintainer"} . " needs to fix the definitions of " .
-                 "moved-default-product. \n", "REOPEN", $exporter);
-    my $def_component = new Bugzilla::Component(
-        {
-            product => $def_product,
-            name    => $params->{"moved-default-component"}
-        })
-    || Error("an invalid default component was defined for the target DB. " .
-             $params->{"maintainer"} . " needs to fix the definitions of " .
-             "moved-default-component.\n", "REOPEN", $exporter);
 }
     
 
@@ -430,6 +405,8 @@ sub process_bug {
     my $exporter_login   = $root->{'att'}->{'exporter'};
     my $exporter         = new Bugzilla::User({ name => $exporter_login });
     my $urlbase          = $root->{'att'}->{'urlbase'};
+    my $url              = $urlbase . $bug_page;
+    trick_taint($url);
 
     # We will store output information in this variable.
     my $log = "";
@@ -500,21 +477,17 @@ sub process_bug {
         }
     }
 
-    my @long_descs;
-    my $private = 0;
-
     # Parse long descriptions
+    my @long_descs;
     foreach my $comment ( $bug->children('long_desc') ) {
         Debug( "Parsing Long Description", DEBUG_LEVEL );
-        my %long_desc;
-        $long_desc{'who'}       = $comment->field('who');
-        $long_desc{'bug_when'}  = $comment->field('bug_when');
-        $long_desc{'isprivate'} = $comment->{'att'}->{'isprivate'} || 0;
+        my %long_desc = ( who       => $comment->field('who'),
+                          bug_when  => format_time($comment->field('bug_when'), '%Y-%m-%d %T'),
+                          isprivate => $comment->{'att'}->{'isprivate'} || 0 );
+
+        # If the exporter is not in the insidergroup, keep the comment public.
+        $long_desc{isprivate} = 0 unless $exporter->is_insider;
 
-        # if one of the comments is private we need to set this flag
-        if ( $long_desc{'isprivate'} && $exporter->is_insider) {
-            $private = 1;
-        }
         my $data = $comment->field('thetext');
         if ( defined $comment->first_child('thetext')->{'att'}->{'encoding'}
             && $comment->first_child('thetext')->{'att'}->{'encoding'} =~
@@ -534,47 +507,27 @@ sub process_bug {
         # Same goes for bug #'s Since we don't know if the referenced bug
         # is also being moved, lets make sure they know it means a different
         # bugzilla.
-        my $url = $urlbase . "show_bug.cgi?id=";
         $data =~ s/([Bb]ugs?\s*\#?\s*(\d+))/$url$2/g;
 
+        # Keep the original commenter if possible, else we will fall back
+        # to the exporter account.
+        $long_desc{whoid} = login_to_id($long_desc{who});
+
+        if (!$long_desc{whoid}) {
+            $data = "The original author of this comment is $long_desc{who}.\n\n" . $data;
+        }
+
         $long_desc{'thetext'} = $data;
         push @long_descs, \%long_desc;
     }
 
-    # instead of giving each comment its own item in the longdescs
-    # table like it should have, lets cat them all into one big
-    # comment otherwise we would have to lie often about who
-    # authored the comment since commenters in one bugzilla probably
-    # don't have accounts in the other one.
-    # If one of the comments is private the whole comment will be
-    # private since we don't want to expose these unnecessarily
-    sub by_date { my @a; my @b; $a->{'bug_when'} cmp $b->{'bug_when'}; }
-    my @sorted_descs     = sort by_date @long_descs;
-    my $long_description = "";
-    for ( my $z = 0 ; $z <= $#sorted_descs ; $z++ ) {
-        if ( $z == 0 ) {
-            $long_description .= "\n\n\n---- Reported by ";
-        }
-        else {
-            $long_description .= "\n\n\n---- Additional Comments From ";
-        }
-        $long_description .= "$sorted_descs[$z]->{'who'} ";
-        $long_description .= "$sorted_descs[$z]->{'bug_when'}";
-        $long_description .= " ----";
-        $long_description .= "\n\n";
-        $long_description .= "THIS COMMENT IS PRIVATE \n"
-          if ( $sorted_descs[$z]->{'isprivate'} );
-        $long_description .= $sorted_descs[$z]->{'thetext'};
-        $long_description .= "\n";
-    }
+    my @sorted_descs = sort { $a->{'bug_when'} cmp $b->{'bug_when'} } @long_descs;
 
-    my $comments;
-
-    $comments .= "\n\n--- Bug imported by $exporter_login ";
+    my $comments = "\n\n--- Bug imported by $exporter_login ";
     $comments .= format_time(scalar localtime(time()), '%Y-%m-%d %R %Z') . " ";
     $comments .= " ---\n\n";
     $comments .= "This bug was previously known as _bug_ $bug_fields{'bug_id'} at ";
-    $comments .= $urlbase . "show_bug.cgi?id=" . $bug_fields{'bug_id'} . "\n";
+    $comments .= $url . $bug_fields{'bug_id'} . "\n";
     if ( defined $bug_fields{'dependson'} ) {
         $comments .= "This bug depended on bug(s) " .
                      join(' ', _to_array($bug_fields{'dependson'})) . ".\n";
@@ -619,12 +572,12 @@ sub process_bug {
     # Timestamps
     push( @query, "creation_ts" );
     push( @values,
-        format_time( $bug_fields{'creation_ts'}, "%Y-%m-%d %X" )
+        format_time( $bug_fields{'creation_ts'}, "%Y-%m-%d %T" )
           || $timestamp );
 
     push( @query, "delta_ts" );
     push( @values,
-        format_time( $bug_fields{'delta_ts'}, "%Y-%m-%d %X" )
+        format_time( $bug_fields{'delta_ts'}, "%Y-%m-%d %T" )
           || $timestamp );
 
     # Bug Access
@@ -634,49 +587,34 @@ sub process_bug {
     push( @query,  "reporter_accessible" );
     push( @values, $bug_fields{'reporter_accessible'} ? 1 : 0 );
 
-    # Product and Component if there is no valid default product and
-    # component defined in the parameters, we wouldn't be here
-    my $def_product =
-      new Bugzilla::Product( { name => $params->{"moved-default-product"} } );
-    my $def_component = new Bugzilla::Component(
-        {
-            product => $def_product,
-            name    => $params->{"moved-default-component"}
-        }
-    );
-    my $product;
-    my $component;
+    my $product = new Bugzilla::Product(
+        { name => $bug_fields{'product'} || '' });
+    if (!$product) {
+        $err .= "Unknown Product " . $bug_fields{'product'} . "\n";
+        $err .= "   Using default product set at the command line.\n";
+        $product = new Bugzilla::Product({ name => $default_product_name })
+            or Error("an invalid default product was defined for the target"
+                     . " DB. " . $params->{"maintainer"} . " needs to specify "
+                     . "--product when calling importxml.pl", "REOPEN", 
+                     $exporter);
+    }
+    my $component = new Bugzilla::Component({
+        product => $product, name => $bug_fields{'component'} || '' });
+    if (!$component) {
+        $err .= "Unknown Component " . $bug_fields{'component'} . "\n";
+        $err .= "   Using default product and component set ";
+        $err .= "at the command line.\n";
 
-    if ( defined $bug_fields{'product'} ) {
-        $product = new Bugzilla::Product( { name => $bug_fields{'product'} } );
-        unless ($product) {
-            $product = $def_product;
-            $err .= "Unknown Product " . $bug_fields{'product'} . "\n";
-            $err .= "   Using default product set in Parameters \n";
+        $product = new Bugzilla::Product({ name => $default_product_name });
+        $component = new Bugzilla::Component({
+           name => $default_component_name, product => $product });
+        if (!$component) {
+            Error("an invalid default component was defined for the target" 
+                  . " DB. ".  $params->{"maintainer"} . " needs to specify " 
+                  . "--component when calling importxml.pl", "REOPEN", 
+                  $exporter);
         }
     }
-    else {
-        $product = $def_product;
-    }
-    if ( defined $bug_fields{'component'} ) {
-        $component = new Bugzilla::Component(
-            {
-                product => $product,
-                name    => $bug_fields{'component'}
-            }
-        );
-        unless ($component) {
-            $component = $def_component;
-            $product   = $def_product;
-            $err .= "Unknown Component " . $bug_fields{'component'} . "\n";
-            $err .= "   Using default product and component set ";
-            $err .= "in Parameters \n";
-        }
-    }
-    else {
-        $component = $def_component;
-        $product   = $def_product;
-    }
 
     my $prod_id = $product->id;
     my $comp_id = $component->id;
@@ -809,7 +747,7 @@ sub process_bug {
         push( @query,  "deadline" );
         if ( defined $bug_fields{'estimated_time'} ) {
             eval {
-                Bugzilla::Bug::ValidateTime($bug_fields{'estimated_time'}, "e");
+                Bugzilla::Object::_validate_time($bug_fields{'estimated_time'}, "e");
             };
             if (!$@){
                 push( @values, $bug_fields{'estimated_time'} );
@@ -818,7 +756,7 @@ sub process_bug {
         }
         if ( defined $bug_fields{'remaining_time'} ) {
             eval {
-                Bugzilla::Bug::ValidateTime($bug_fields{'remaining_time'}, "r");
+                Bugzilla::Object::_validate_time($bug_fields{'remaining_time'}, "r");
             };
             if (!$@){
                 push( @values, $bug_fields{'remaining_time'} );
@@ -827,7 +765,7 @@ sub process_bug {
         }
         if ( defined $bug_fields{'actual_time'} ) {
             eval {
-                Bugzilla::Bug::ValidateTime($bug_fields{'actual_time'}, "a");
+                Bugzilla::Object::_validate_time($bug_fields{'actual_time'}, "a");
             };
             if ($@){
                 $bug_fields{'actual_time'} = 0.0;
@@ -890,8 +828,10 @@ sub process_bug {
             push( @values, $qa_contact );
         }
         else {
-            push( @values, $component->default_qa_contact->id || undef );
-            if ($component->default_qa_contact->id){
+            push(@values, $component->default_qa_contact ?
+                          $component->default_qa_contact->id : undef);
+
+            if ($component->default_qa_contact) {
                 $err .= "Setting qa contact to the default for this product.\n";
                 $err .= "   This bug either had no qa contact or an invalid one.\n";
             }
@@ -899,15 +839,12 @@ sub process_bug {
     }
 
     # Status & Resolution
-    my $has_res = defined($bug_fields{'resolution'});
-    my $has_status = defined($bug_fields{'bug_status'});
     my $valid_res = check_field('resolution',  
                                   scalar $bug_fields{'resolution'}, 
                                   undef, ERR_LEVEL );
     my $valid_status = check_field('bug_status',  
                                   scalar $bug_fields{'bug_status'}, 
                                   undef, ERR_LEVEL );
-    my $is_open = is_open_state($bug_fields{'bug_status'}); 
     my $status = $bug_fields{'bug_status'} || undef;
     my $resolution = $bug_fields{'resolution'} || undef;
     
@@ -927,9 +864,9 @@ sub process_bug {
     # that might not yet be in the database we have no way of populating
     # this table. Change the resolution instead.
     if ( $valid_res  && ( $bug_fields{'resolution'} eq "DUPLICATE" ) ) {
-        $resolution = "MOVED";
+        $resolution = "INVALID";
         $err .= "This bug was marked DUPLICATE in the database ";
-        $err .= "it was moved from.\n    Changing resolution to \"MOVED\"\n";
+        $err .= "it was moved from.\n    Changing resolution to \"INVALID\"\n";
     } 
 
     # If there is at least 1 initial bug status different from UNCO, use it,
@@ -955,10 +892,10 @@ sub process_bug {
         }
     }
 
-    if($has_status){
+    if ($status) {
         if($valid_status){
-            if($is_open){
-                if($has_res){
+            if (is_open_state($status)) {
+                if ($resolution) {
                     $err .= "Resolution set on an open status.\n";
                     $err .= "   Dropping resolution $resolution\n";
                     $resolution = undef;
@@ -980,7 +917,6 @@ sub process_bug {
                     if($status eq "UNCONFIRMED"){
                         $err .= "Bug Status was UNCONFIRMED but everconfirmed was true\n";
                         $err .= "   Setting status to $initial_status\n";
-                        $err .= "Resetting votes to 0\n" if ( $bug_fields{'votes'} );
                         $status = $initial_status;
                     }
                 }
@@ -992,8 +928,8 @@ sub process_bug {
                     }
                 }
             }
-            else{ # $is_open is false
-               if(!$has_res){
+            else {
+               if (!$resolution) {
                    $err .= "Missing Resolution. Setting status to ";
                    if($everconfirmed){
                        $status = $initial_status;
@@ -1004,10 +940,10 @@ sub process_bug {
                        $err .= "UNCONFIRMED\n";
                    }
                }
-               if(!$valid_res){
+               elsif (!$valid_res) {
                    $err .= "Unknown resolution \"$resolution\".\n";
-                   $err .= "   Setting resolution to MOVED\n";
-                   $resolution = "MOVED";
+                   $err .= "   Setting resolution to INVALID\n";
+                   $resolution = "INVALID";
                }
             }   
         }
@@ -1023,9 +959,8 @@ sub process_bug {
             $err .=  $bug_fields{'bug_status'} . "\".\n";
             $resolution = undef;
         }
-                
     }
-    else{ #has_status is false
+    else {
         if($everconfirmed){  
             $status = $initial_status;
         }
@@ -1036,8 +971,8 @@ sub process_bug {
         $err .= "   Previous status was unknown\n";
         $resolution = undef;
     }
-                                 
-    if (defined $resolution){
+
+    if ($resolution) {
         push( @query,  "resolution" );
         push( @values, $resolution );
     }
@@ -1109,6 +1044,7 @@ sub process_bug {
 
     $dbh->do( $query, undef, @values );
     my $id = $dbh->bz_last_key( 'bugs', 'bug_id' );
+    my $bug_obj = Bugzilla::Bug->new($id);
 
     # We are almost certain to get some uninitialized warnings
     # Since this is just for debugging the query, let's shut them up
@@ -1161,15 +1097,6 @@ sub process_bug {
                 $keywordseen{$keyword_obj->id} = 1;
             }
         }
-        my ($keywordarray) = $dbh->selectcol_arrayref(
-            "SELECT d.name FROM keyworddefs d
-                    INNER JOIN keywords k 
-                    ON d.id = k.keywordid 
-                    WHERE k.bug_id = ? 
-                    ORDER BY d.name", undef, $id);
-        my $keywordstring = join( ", ", @{$keywordarray} );
-        $dbh->do( "UPDATE bugs SET keywords = ? WHERE bug_id = ?",
-            undef, $keywordstring, $id )
     }
 
     # Insert values of custom multi-select fields. They have already
@@ -1200,31 +1127,41 @@ sub process_bug {
             $err .= "No attachment ID specified, dropping attachment\n";
             next;
         }
-        if (!$exporter->is_insider && $att->{'isprivate'}) {
-            $err .= "Exporter not in insidergroup and attachment marked private.\n";
+
+        my $attacher;
+        if ($att->{'attacher'}) {
+            $attacher = Bugzilla::User->new({name => $att->{'attacher'}, cache => 1});
+        }
+        my $new_attacher = $attacher || $exporter;
+
+        if ($att->{'isprivate'} && !$new_attacher->is_insider) {
+            my $who = $new_attacher->login;
+            $err .= "$who not in insidergroup and attachment marked private.\n";
             $err .= "   Marking attachment public\n";
             $att->{'isprivate'} = 0;
         }
 
-        my $attacher_id = $att->{'attacher'} ? login_to_id($att->{'attacher'}) : undef;
+        # We log in the user so that the attachment creator is set correctly.
+        Bugzilla->set_user($new_attacher);
 
-        $dbh->do("INSERT INTO attachments 
-                 (bug_id, creation_ts, modification_time, filename, description,
-                 mimetype, ispatch, isprivate, isobsolete, submitter_id) 
-                 VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?)",
-            undef, $id, $att->{'date'}, $att->{'date'}, $att->{'filename'},
-            $att->{'desc'}, $att->{'ctype'}, $att->{'ispatch'},
-            $att->{'isprivate'}, $att->{'isobsolete'}, $attacher_id || $exporterid);
-        my $att_id   = $dbh->bz_last_key( 'attachments', 'attach_id' );
-        my $att_data = $att->{'data'};
-        my $sth = $dbh->prepare("INSERT INTO attach_data (id, thedata) 
-                                 VALUES ($att_id, ?)" );
-        trick_taint($att_data);
-        $sth->bind_param( 1, $att_data, $dbh->BLOB_TYPE );
-        $sth->execute();
+        my $attachment = Bugzilla::Attachment->create(
+            { bug           => $bug_obj,
+              creation_ts   => $att->{date},
+              data          => $att->{data},
+              description   => $att->{desc},
+              filename      => $att->{filename},
+              ispatch       => $att->{ispatch},
+              isprivate     => $att->{isprivate},
+              isobsolete    => $att->{isobsolete},
+              mimetype      => $att->{ctype},
+            });
+        my $att_id = $attachment->id;
+
+        # We log out the attacher as the remaining steps are not on his behalf.
+        Bugzilla->logout_request;
 
         $comments .= "Imported an attachment (id=$att_id)\n";
-        if (!$attacher_id) {
+        if (!$attacher) {
             if ($att->{'attacher'}) {
                 $err .= "The original submitter of attachment $att_id was\n   ";
                 $err .= $att->{'attacher'} . ", but he doesn't have an account here.\n";
@@ -1251,20 +1188,22 @@ sub process_bug {
     # Clear the attachments array for the next bug
     @attachments = ();
 
-    # Insert longdesc and append any errors
+    # Insert comments and append any errors
     my $worktime = $bug_fields{'actual_time'} || 0.0;
     $worktime = 0.0 if (!$exporter->is_timetracker);
-    $long_description .= "\n" . $comments;
-    if ($err) {
-        $long_description .= "\n$err\n";
+    $comments .= "\n$err\n" if $err;
+
+    my $sth_comment =
+      $dbh->prepare('INSERT INTO longdescs (bug_id, who, bug_when, isprivate,
+                                            thetext, work_time)
+                     VALUES (?, ?, ?, ?, ?, ?)');
+
+    foreach my $c (@sorted_descs) {
+        $sth_comment->execute($id, $c->{whoid} || $exporterid, $c->{bug_when},
+                              $c->{isprivate}, $c->{thetext}, 0);
     }
-    trick_taint($long_description);
-    $dbh->do("INSERT INTO longdescs 
-                     (bug_id, who, bug_when, work_time, isprivate, thetext) 
-                     VALUES (?,?,?,?,?,?)", undef,
-        $id, $exporterid, $timestamp, $worktime, $private, $long_description
-    );
-    Bugzilla::Bug->new($id)->_sync_fulltext('new_bug');
+    $sth_comment->execute($id, $exporterid, $timestamp, 0, $comments, $worktime);
+    Bugzilla::Bug->new($id)->_sync_fulltext( new_bug => 1);
 
     # Add this bug to each group of which its product is a member.
     my $sth_group = $dbh->prepare("INSERT INTO bug_group_map (bug_id, group_id) 
@@ -1276,7 +1215,7 @@ sub process_bug {
         }
     }
 
-    $log .= "Bug ${urlbase}show_bug.cgi?id=$bug_fields{'bug_id'} ";
+    $log .= "Bug ${url}$bug_fields{'bug_id'} ";
     $log .= "imported as bug $id.\n";
     $log .= $params->{"urlbase"} . "show_bug.cgi?id=$id\n\n";
     if ($err) {
@@ -1286,7 +1225,7 @@ sub process_bug {
     }
     Debug( $log, OK_LEVEL );
     push(@logs, $log);
-    Bugzilla::BugMail::Send( $id, { 'changer' => $exporter_login } ) if ($mail);
+    Bugzilla::BugMail::Send( $id, { 'changer' => $exporter } ) if ($mail);
 
     # done with the xml data. Lets clear it from memory
     $twig->purge;
@@ -1347,31 +1286,45 @@ importxml - Import bugzilla bug data from xml.
 
 =head1 SYNOPSIS
 
-    importxml.pl [options] [file ...]
-
- Options:
-       -? --help        brief help message
-       -v --verbose     print error and debug information. 
-                        Multiple -v increases verbosity
-       -m --sendmail    send mail to recipients with log of bugs imported
-       --attach_path    The path to the attachment files.
-                        (Required if encoding="filename" is used for attachments.)
+ importxml.pl [options] [file ...]
 
 =head1 OPTIONS
 
-=over 8
+=over
 
 =item B<-?>
 
-    Print a brief help message and exits.
+Print a brief help message and exit.
 
 =item B<-v>
 
-    Print error and debug information. Mulltiple -v increases verbosity
+Print error and debug information. Mulltiple -v increases verbosity
 
-=item B<-m>
+=item B<-m> B<--sendmail>
 
-    Send mail to exporter with a log of bugs imported and any errors.
+Send mail to exporter with a log of bugs imported and any errors.
+
+=item B<--attach_path>
+
+The path to the attachment files. (Required if encoding="filename"
+is used for attachments.)
+
+=item B<--bug_page>
+
+The page that links to the bug on top of urlbase. Its default value
+is "show_bug.cgi?id=", which is what Bugzilla installations use.
+You only need to pass this argument if you are importing bugs from
+another bug tracking system.
+
+=item B<--product=name>
+
+The product to put the bug in if the product specified in the
+XML doesn't exist.
+
+=item B<--component=name>
+
+The component to put the bug in if the component specified in the
+XML doesn't exist.
 
 =back
 
@@ -1397,4 +1350,3 @@ importxml - Import bugzilla bug data from xml.
      importxml.pl [options] bugsfile.xml
 
 =cut
-
