@@ -1,24 +1,9 @@
-/* The contents of this file are subject to the Mozilla Public
- * License Version 1.1 (the "License"); you may not use this file
- * except in compliance with the License. You may obtain a copy of
- * the License at http://www.mozilla.org/MPL/
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  *
- * Software distributed under the License is distributed on an "AS
- * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
- * implied. See the License for the specific language governing
- * rights and limitations under the License.
- *
- * The Original Code is the Bugzilla Bug Tracking System.
- *
- * The Initial Developer of the Original Code is Netscape Communications
- * Corporation. Portions created by Netscape are
- * Copyright (C) 1998 Netscape Communications Corporation. All
- * Rights Reserved.
- *
- * Contributor(s): Myk Melez <myk@mozilla.org>
- *                 Joel Peshkin <bugreport@peshkin.net>
- *                 Erik Stambaugh <erik@dasbistro.com>
- *                 Marc Schumann <wurblzap@gmail.com>
+ * This Source Code Form is "Incompatible With Secondary Licenses", as
+ * defined by the Mozilla Public License, v. 2.0.
  */
 
 function validateAttachmentForm(theform) {
@@ -39,27 +24,24 @@ function updateCommentPrivacy(checkbox) {
     }
 }
 
-function setContentTypeDisabledState(form)
-{
+function setContentTypeDisabledState(form) {
     var isdisabled = false;
     if (form.ispatch.checked)
         isdisabled = true;
 
-    for (var i=0 ; i<form.contenttypemethod.length ; i++)
+    for (var i = 0; i < form.contenttypemethod.length; i++)
         form.contenttypemethod[i].disabled = isdisabled;
 
     form.contenttypeselection.disabled = isdisabled;
     form.contenttypeentry.disabled = isdisabled;
 }
 
-function URLFieldHandler() {
-    var field_attachurl = document.getElementById("attachurl");
-    var greyfields = new Array("data", "ispatch", "autodetect",
-                               "list", "manual", "bigfile",
-                               "contenttypeselection",
-                               "contenttypeentry");
+function TextFieldHandler() {
+    var field_text = document.getElementById("attach_text");
+    var greyfields = new Array("data", "autodetect", "list", "manual",
+                               "contenttypeselection", "contenttypeentry");
     var i, thisfield;
-    if (field_attachurl.value.match(/^\s*$/)) {
+    if (field_text.value.match(/^\s*$/)) {
         for (i = 0; i < greyfields.length; i++) {
             thisfield = document.getElementById(greyfields[i]);
             if (thisfield) {
@@ -78,7 +60,7 @@ function URLFieldHandler() {
 
 function DataFieldHandler() {
     var field_data = document.getElementById("data");
-    var greyfields = new Array("attachurl");
+    var greyfields = new Array("attach_text");
     var i, thisfield;
     if (field_data.value.match(/^\s*$/)) {
         for (i = 0; i < greyfields.length; i++) {
@@ -102,11 +84,9 @@ function clearAttachmentFields() {
 
     document.getElementById('data').value = '';
     DataFieldHandler();
-    if ((element = document.getElementById('bigfile')))
-        element.checked = '';
-    if ((element = document.getElementById('attachurl'))) {
+    if ((element = document.getElementById('attach_text'))) {
         element.value = '';
-        URLFieldHandler();
+        TextFieldHandler();
     }
     document.getElementById('description').value = '';
     /* Fire onchange so that the disabled state of the content-type
@@ -307,16 +287,15 @@ function hideElementById(id)
 {
   var elm = document.getElementById(id);
   if (elm) {
-    elm.style.display = 'none';
+    YAHOO.util.Dom.addClass(elm, 'bz_default_hidden');
   }
 }
 
-function showElementById(id, val)
+function showElementById(id)
 {
   var elm = document.getElementById(id);
   if (elm) {
-    if (!val) val = 'inline';
-    elm.style.display = val;
+    YAHOO.util.Dom.removeClass(elm, 'bz_default_hidden');
   }
 }
 
@@ -327,12 +306,37 @@ function normalizeComments()
 
   var small = document.getElementById('smallCommentFrame');
   var big = document.getElementById('editFrame');
-  if ( (small) && (small.style.display == 'none') )
+  if ( (small) && YAHOO.util.Dom.hasClass(small, 'bz_default_hidden') )
   {
     small.parentNode.removeChild(small);
   }
-  if ( (big) && (big.style.display == 'none') )
+  if ( (big) && YAHOO.util.Dom.hasClass(big, 'bz_default_hidden') )
   {
     big.parentNode.removeChild(big);
   }
 }
+
+function toggle_attachment_details_visibility ( ) 
+{
+    // show hide classes
+    var container = document.getElementById('attachment_info');
+    if( YAHOO.util.Dom.hasClass(container, 'read') ){
+        YAHOO.util.Dom.replaceClass(container, 'read', 'edit');
+    }else{
+        YAHOO.util.Dom.replaceClass(container, 'edit', 'read');
+    }
+}
+
+/* Used in bug/create.html.tmpl to show/hide the attachment field. */
+
+function handleWantsAttachment(wants_attachment) {
+    if (wants_attachment) {
+        hideElementById('attachment_false');
+        showElementById('attachment_true');
+    }
+    else {
+        showElementById('attachment_false');
+        hideElementById('attachment_true');
+        clearAttachmentFields();
+    }
+}
