@@ -1,26 +1,9 @@
-# -*- Mode: perl; indent-tabs-mode: nil -*-
-#
-# The contents of this file are subject to the Mozilla Public
-# License Version 1.1 (the "License"); you may not use this file
-# except in compliance with the License. You may obtain a copy of
-# the License at http://www.mozilla.org/MPL/
-#
-# Software distributed under the License is distributed on an "AS
-# IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
-# implied. See the License for the specific language governing
-# rights and limitations under the License.
-#
-# The Original Code are the Bugzilla tests.
-#
-# The Initial Developer of the Original Code is Jacob Steenhagen.
-# Portions created by Jacob Steenhagen are
-# Copyright (C) 2001 Jacob Steenhagen. All
-# Rights Reserved.
-#
-# Contributor(s): Jacob Steenhagen <jake@bugzilla.org>
-#                 David D. Kilzer <ddkilzer@kilzer.net>
-#                 Tobias Burnus <burnus@net-b.de>
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at http://mozilla.org/MPL/2.0/.
 #
+# This Source Code Form is "Incompatible With Secondary Licenses", as
+# defined by the Mozilla Public License, v. 2.0.
 
 package Support::Templates;
 
@@ -29,11 +12,14 @@ use strict;
 use lib 't';
 use base qw(Exporter);
 @Support::Templates::EXPORT = 
-         qw(@languages @include_paths %include_path @referenced_files 
-            %actual_files $num_actual_files);
-use vars qw(@languages @include_paths %include_path @referenced_files 
-            %actual_files $num_actual_files);
+         qw(@languages @include_paths @english_default_include_paths
+         %include_path @referenced_files %actual_files $num_actual_files);
+use vars qw(@languages @include_paths @english_default_include_paths
+            %include_path @referenced_files %actual_files $num_actual_files);
 
+use Bugzilla;
+use Bugzilla::Constants;
+use Bugzilla::Install::Util qw(template_include_path);
 use Support::Files;
 
 use File::Find;
@@ -48,6 +34,18 @@ use File::Spec;
 # All include paths
 @include_paths = ();
 
+# English default include paths
+push @english_default_include_paths,
+    File::Spec->catdir(bz_locations()->{'templatedir'}, 'en', 'default');
+
+# And the extensions too
+foreach my $extension (@Support::Files::extensions) {
+    my $dir = File::Spec->catdir($extension, 'template', 'en', 'default');
+    if (-e $dir) {
+        push @english_default_include_paths, $dir;
+    }
+}
+
 # Files which are referenced in the cgi files
 @referenced_files = ();
 
@@ -57,30 +55,9 @@ use File::Spec;
 # total number of actual_files
 $num_actual_files = 0;
 
-# Scan for the template available languages and include paths
-{
-    opendir(DIR, "template") || die "Can't open  'template': $!";
-    my @files = grep { /^[a-z-]+$/i } readdir(DIR);
-    closedir DIR;
-
-    foreach my $langdir (@files) {
-        next if($langdir =~ /^CVS$/i);
-
-        my $path = File::Spec->catdir('template', $langdir, 'custom');
-        my @dirs = ();
-        push(@dirs, $path) if(-d $path);
-        $path = File::Spec->catdir('template', $langdir, 'extension');
-        push(@dirs, $path) if(-d $path);
-        $path = File::Spec->catdir('template', $langdir, 'default');
-        push(@dirs, $path) if(-d $path);
-
-        next if(scalar(@dirs) == 0);
-        push(@languages, $langdir);
-        push(@include_paths, @dirs);
-        $include_path{$langdir} = join(":",@dirs);
-    }
-}
-
+# Set the template available languages and include paths
+@languages = @{ Bugzilla->languages };
+@include_paths = @{ template_include_path({ language => Bugzilla->languages }) };
 
 my @files;
 
