@@ -1,23 +1,9 @@
-# -*- Mode: perl; indent-tabs-mode: nil -*-
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at http://mozilla.org/MPL/2.0/.
 #
-# The contents of this file are subject to the Mozilla Public
-# License Version 1.1 (the "License"); you may not use this file
-# except in compliance with the License. You may obtain a copy of
-# the License at http://www.mozilla.org/MPL/
-#
-# Software distributed under the License is distributed on an "AS
-# IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
-# implied. See the License for the specific language governing
-# rights and limitations under the License.
-#
-# The Original Code are the Bugzilla tests.
-#
-# The Initial Developer of the Original Code is Jacob Steenhagen.
-# Portions created by Jacob Steenhagen are
-# Copyright (C) 2001 Jacob Steenhagen. All
-# Rights Reserved.
-#
-# Contributor(s): Gervase Markham <gerv@gerv.net>
+# This Source Code Form is "Incompatible With Secondary Licenses", as
+# defined by the Mozilla Public License, v. 2.0.
 
 # Important! The following classes of directives are excluded in the test,
 # and so do not need to be added here. Doing so will cause warnings.
@@ -34,7 +20,7 @@
 #                                   [% foo.push() %]
 # TT loop variables               - [% loop.count %]
 # Already-filtered stuff          - [% wibble FILTER html %]
-#   where the filter is one of html|csv|js|url_quote|quoteUrls|time|uri|xml|none
+#   where the filter is one of html|csv|js|quoteUrls|time|uri|xml|none
 
 %::safe = (
 
@@ -52,28 +38,13 @@
 ],
 
 'flag/list.html.tmpl' => [
-  'flag.id', 
   'flag.status', 
   'type.id', 
 ],
 
-'search/boolean-charts.html.tmpl' => [
-  '"field${chartnum}-${rownum}-${colnum}"', 
-  '"value${chartnum}-${rownum}-${colnum}"', 
-  'field.name', 
-  '"${chartnum}-${rownum}-${newor}"', 
-  '"${chartnum}-${newand}-0"', 
-  'newchart',
-  'jsmagic',
-],
-
 'search/form.html.tmpl' => [
   'qv.name',
   'qv.description',
-  'field.name',
-  'field.description',
-  'field.accesskey',
-  'sel.name',
 ],
 
 'search/search-specific.html.tmpl' => [
@@ -102,7 +73,6 @@
 ],
 
 'reports/report-table.html.tmpl' => [
-  '"&amp;$tbl_vals" IF tbl_vals', 
   '"&amp;$col_vals" IF col_vals', 
   '"&amp;$row_vals" IF row_vals', 
   'classes.$row_idx.$col_idx', 
@@ -119,9 +89,7 @@
   'imageurl', 
   'formaturl', 
   'other_format.name', 
-  'sizeurl', 
   'switchbase',
-  'format',
   'cumulate',
 ],
 
@@ -169,7 +137,6 @@
 'list/table.html.tmpl' => [
   'tableheader',
   'bug.bug_id', 
-  'abbrev.$id.title || field_descs.$id || column.title',
 ],
 
 'list/list.csv.tmpl' => [
@@ -207,10 +174,6 @@
   'series.frequency * 2',
 ],
 
-'global/per-bug-queries.html.tmpl' => [
-  '" value=\"$bugids\"" IF bugids',
-],
-
 'global/select-menu.html.tmpl' => [
   'options', 
   'size', 
@@ -228,16 +191,15 @@
 
 'global/confirm-user-match.html.tmpl' => [
   'script',
-  'fields.${field_name}.flag_type.name',
 ],
 
 'global/site-navigation.html.tmpl' => [
   'bug.bug_id', 
-  'bug.votes', 
 ],
 
 'bug/comments.html.tmpl' => [
   'comment.id',
+  'comment.count',
   'bug.bug_id',
 ],
 
@@ -260,17 +222,11 @@
 ],
 
 'bug/edit.html.tmpl' => [
-  'bug.deadline',
   'bug.remaining_time', 
   'bug.delta_ts', 
   'bug.bug_id', 
-  'bug.votes', 
   'group.bit', 
-  'dep.title', 
-  'dep.fieldname', 
-  'bug.${dep.fieldname}.join(\', \')', 
   'selname',
-  '" accesskey=\"$accesskey\"" IF accesskey',
   'inputname',
   '" colspan=\"$colspan\"" IF colspan',
   '" size=\"$size\"" IF size',
@@ -312,19 +268,6 @@
        FILTER format("%d")', 
 ],
 
-'bug/votes/list-for-bug.html.tmpl' => [
-  'voter.vote_count', 
-  'total', 
-],
-
-'bug/votes/list-for-user.html.tmpl' => [
-  'product.maxperbug', 
-  'bug.id', 
-  'bug.count', 
-  'product.total', 
-  'product.maxvotes', 
-],
-
 'bug/process/results.html.tmpl' => [
   'title.$type', 
   '"$terms.Bug $id" FILTER bug_link(id)',
@@ -332,9 +275,6 @@
 ],
 
 'bug/create/create.html.tmpl' => [
-  'g.bit',
-  'sel.name',
-  'sel.description',
   'cloned_bug_id',
 ],
 
@@ -361,7 +301,6 @@
 'attachment/edit.html.tmpl' => [
   'attachment.id', 
   'attachment.bug_id', 
-  'a',
   'editable_or_hide',
 ],
 
@@ -410,12 +349,13 @@
 ],
 
 'admin/table.html.tmpl' => [
-  'link_uri'
+  'contentlink'
 ],
 
 'admin/custom_fields/cf-js.js.tmpl' => [
   'constants.FIELD_TYPE_SINGLE_SELECT',
   'constants.FIELD_TYPE_MULTI_SELECT',
+  'constants.FIELD_TYPE_BUG_ID',
 ],
 
 'admin/params/common.html.tmpl' => [
@@ -449,12 +389,6 @@
 ],
 
 'admin/flag-type/edit.html.tmpl' => [
-  'action', 
-  'type.id', 
-  'type.target_type', 
-  'type.sortkey || 1',
-  'typeLabelLowerPlural',
-  'typeLabelLowerSingular',
   'selname',
 ],
 
@@ -481,7 +415,6 @@
   'flags.setter',
   'longdescs',
   'quips',
-  'votes',
   'series',
   'watch.watched',
   'watch.watcher',
@@ -528,4 +461,8 @@
   'group.id',
 ],
 
+'config.rdf.tmpl' => [
+  'escaped_urlbase',
+],
+
 );
