@@ -1,18 +1,10 @@
 #!/usr/bin/perl -w
-# -*- Mode: perl; indent-tabs-mode: nil -*-
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at http://mozilla.org/MPL/2.0/.
 #
-# The contents of this file are subject to the Mozilla Public
-# License Version 1.1 (the "License"); you may not use this file
-# except in compliance with the License. You may obtain a copy of
-# the License at http://www.mozilla.org/MPL/
-#
-# Software distributed under the License is distributed on an "AS
-# IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
-# implied. See the License for the specific language governing
-# rights and limitations under the License.
-#
-# Contributor(s): Joel Peshkin <bugreport@peshkin.net>
-#                 Byron Jones <byron@glob.com.au>
+# This Source Code Form is "Incompatible With Secondary Licenses", as
+# defined by the Mozilla Public License, v. 2.0.
 
 # testserver.pl is invoked with the baseurl of the Bugzilla installation
 # as its only argument.  It attempts to troubleshoot as many installation
@@ -23,6 +15,7 @@ use lib qw(. lib);
 
 use Bugzilla;
 use Bugzilla::Constants;
+use Bugzilla::Util;
 
 use Socket;
 
@@ -31,10 +24,10 @@ my $datadir = bz_locations()->{'datadir'};
 eval "require LWP; require LWP::UserAgent;";
 my $lwp = $@ ? 0 : 1;
 
-if ((@ARGV != 1) || ($ARGV[0] !~ /^https?:/))
+if ((@ARGV != 1) || ($ARGV[0] !~ /^https?:/i))
 {
-    print "Usage: $0 <URL to this Bugzilla installation>\n";
-    print "e.g.:  $0 http://www.mycompany.com/bugzilla\n";
+    say "Usage: $0 <URL to this Bugzilla installation>";
+    say "e.g.:  $0 http://www.mycompany.com/bugzilla";
     exit(1);
 }
 
@@ -67,26 +60,26 @@ else {
 # Check $webservergroup against the server's GID
 if ($sgid > 0) {
     if ($webservergroup eq "") {
-        print 
+        say 
 "WARNING \$webservergroup is set to an empty string.
 That is a very insecure practice. Please refer to the
-Bugzilla documentation.\n";
+Bugzilla documentation.";
     }
     elsif ($webgroupnum == $sgid || Bugzilla->localconfig->{use_suexec}) {
-        print "TEST-OK Webserver is running under group id in \$webservergroup.\n";
+        say "TEST-OK Webserver is running under group id in \$webservergroup.";
     }
     else {
-        print 
+        say 
 "TEST-WARNING Webserver is running under group id not matching \$webservergroup.
 This if the tests below fail, this is probably the problem.
 Please refer to the web server configuration section of the Bugzilla guide. 
-If you are using virtual hosts or suexec, this warning may not apply.\n";
+If you are using virtual hosts or suexec, this warning may not apply.";
     }
 }
 elsif (!ON_WINDOWS) {
-   print
+   say
 "TEST-WARNING Failed to find the GID for the 'httpd' process, unable
-to validate webservergroup.\n";
+to validate webservergroup.";
 }
 
 
@@ -94,26 +87,26 @@ to validate webservergroup.\n";
 $ARGV[0] =~ s/\/$//;
 my $url = $ARGV[0] . "/images/padlock.png";
 if (fetch($url)) {
-    print "TEST-OK Got padlock picture.\n";
+    say "TEST-OK Got padlock picture.";
 } else {
-    print 
+    say 
 "TEST-FAILED Fetch of images/padlock.png failed
 Your web server could not fetch $url.
-Check your web server configuration and try again.\n";
+Check your web server configuration and try again.";
     exit(1);
 }
 
 # Try to execute a cgi script
-my $response = fetch($ARGV[0] . "/testagent.cgi");
+my $response = clean_text(fetch($ARGV[0] . "/testagent.cgi"));
 if ($response =~ /^OK (.*)$/) {
-    print "TEST-OK Webserver is executing CGIs via $1.\n";
+    say "TEST-OK Webserver is executing CGIs via $1.";
 } elsif ($response =~ /^#!/) {
-    print 
+    say
 "TEST-FAILED Webserver is fetching rather than executing CGI files.
-Check the AddHandler statement in your httpd.conf file.\n";
+Check the AddHandler statement in your httpd.conf file.";
     exit(1);
 } else {
-    print "TEST-FAILED Webserver is not executing CGI files.\n"; 
+    say "TEST-FAILED Webserver is not executing CGI files.";
 }
 
 # Make sure that the web server is honoring .htaccess files
@@ -122,13 +115,13 @@ $localconfig =~ s~^\./~~;
 $url = $ARGV[0] . "/$localconfig";
 $response = fetch($url);
 if ($response) {
-    print 
+    say
 "TEST-FAILED Webserver is permitting fetch of $url.
 This is a serious security problem.
-Check your web server configuration.\n";
+Check your web server configuration.";
     exit(1);
 } else {
-    print "TEST-OK Webserver is preventing fetch of $url.\n";
+    say "TEST-OK Webserver is preventing fetch of $url.";
 }
 
 # Test chart generation
@@ -142,8 +135,8 @@ if ($@ eq '') {
         my $gdlib = `gdlib-config --version 2>&1` || "";
         $gdlib =~ s/\n$//;
         if (!$gdlib) {
-            print "TEST-WARNING Failed to run gdlib-config; can't compare " .
-                  "GD versions.\n";
+            say "TEST-WARNING Failed to run gdlib-config; can't compare " .
+                  "GD versions.";
         }
         else {
             my $gd = $GD::VERSION;
@@ -154,9 +147,9 @@ if ($@ eq '') {
             $gd =~ s/^([^\.]+)\..*/$1/;
     
             if ($gdlib == $gd) {
-                print "TEST-OK $verstring; Major versions match.\n";
+                say "TEST-OK $verstring; Major versions match.";
             } else {
-                print "TEST-FAILED $verstring; Major versions do not match.\n";
+                say "TEST-FAILED $verstring; Major versions do not match.";
             }
         }
     }
@@ -177,17 +170,17 @@ if ($@ eq '') {
             create_file("$datadir/testgd-local.png", $image->png);
             check_image("$datadir/testgd-local.png", 'GD');
         } else {
-            print "TEST-FAILED GD doesn't support PNG generation.\n";
+            say "TEST-FAILED GD doesn't support PNG generation.";
         }
     };
     if ($@ ne '') {
-        print "TEST-FAILED GD returned: $@\n";
+        say "TEST-FAILED GD returned: $@";
     }
 
     # Test Chart
     eval 'use Chart::Lines';
     if ($@) {
-        print "TEST-FAILED Chart::Lines is not installed.\n";
+        say "TEST-FAILED Chart::Lines is not installed.";
     } else {
         eval {
             my $chart = Chart::Lines->new(400, 400);
@@ -199,16 +192,16 @@ if ($@ eq '') {
             check_image("$datadir/testchart-local.png", "Chart");
         };
         if ($@ ne '') {
-            print "TEST-FAILED Chart returned: $@\n";
+            say "TEST-FAILED Chart returned: $@";
         }
     }
 
     eval 'use Template::Plugin::GD::Image';
     if ($@) {
-        print "TEST-FAILED Template::Plugin::GD is not installed.\n";
+        say "TEST-FAILED Template::Plugin::GD is not installed.";
     }
     else {
-        print "TEST-OK Template::Plugin::GD is installed.\n";
+        say "TEST-OK Template::Plugin::GD is installed.";
     }
 }
 
@@ -262,10 +255,10 @@ sub check_image {
     my ($local_file, $library) = @_;
     my $filedata = read_file($local_file);
     if ($filedata =~ /^\x89\x50\x4E\x47\x0D\x0A\x1A\x0A/) {
-        print "TEST-OK $library library generated a good PNG image.\n";
+        say "TEST-OK $library library generated a good PNG image.";
         unlink $local_file;
     } else {
-        print "TEST-WARNING $library library did not generate a good PNG.\n";
+        say "TEST-WARNING $library library did not generate a good PNG.";
     }
 }
 
