@@ -1,24 +1,10 @@
 #!/usr/bin/perl -wT
-# -*- Mode: perl; indent-tabs-mode: nil -*-
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at http://mozilla.org/MPL/2.0/.
 #
-# The contents of this file are subject to the Mozilla Public
-# License Version 1.1 (the "License"); you may not use this file
-# except in compliance with the License. You may obtain a copy of
-# the License at http://www.mozilla.org/MPL/
-#
-# Software distributed under the License is distributed on an "AS
-# IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
-# implied. See the License for the specific language governing
-# rights and limitations under the License.
-#
-# The Original Code is the Bugzilla Bug Tracking System.
-#
-# The Initial Developer of the Original Code is Netscape Communications
-# Corporation. Portions created by Netscape are
-# Copyright (C) 1998 Netscape Communications Corporation. All
-# Rights Reserved.
-#
-# Contributor(s): Erik Stambaugh <erik@dasbistro.com>
+# This Source Code Form is "Incompatible With Secondary Licenses", as
+# defined by the Mozilla Public License, v. 2.0.
 
 ################################################################################
 # Script Initialization
@@ -150,20 +136,22 @@ while (my ($schedule_id, $day, $time) = $sched_h->fetchrow_array) {
         # A time greater than now means it still has to run today
         elsif ($time >= $now_hour) {
             # set it to today + number of hours
-            $sth = $dbh->prepare("UPDATE whine_schedules " .
-                                 "SET run_next = CURRENT_DATE + " .
-                                 $dbh->sql_interval('?', 'HOUR') .
-                                 " WHERE id = ?");
+            $sth = $dbh->prepare(
+                "UPDATE whine_schedules " .
+                   "SET run_next = " .
+                        $dbh->sql_date_math('CURRENT_DATE', '+', '?', 'HOUR') .
+                " WHERE id = ?");
             $sth->execute($time, $schedule_id);
         }
         # the target time is less than the current time
         else { # set it for the next applicable day
             $day = &get_next_date($day);
+            my $run_next = $dbh->sql_date_math('(' 
+                . $dbh->sql_date_math('CURRENT_DATE', '+', '?', 'DAY')
+                . ')', '+', '?', 'HOUR');
             $sth = $dbh->prepare("UPDATE whine_schedules " .
-                                 "SET run_next = (CURRENT_DATE + " .
-                                 $dbh->sql_interval('?', 'DAY') . ") + " .
-                                 $dbh->sql_interval('?', 'HOUR') .
-                                 " WHERE id = ?");
+                                    "SET run_next = $run_next
+                                   WHERE id = ?");
             $sth->execute($day, $time, $schedule_id);
         }
 
@@ -176,11 +164,12 @@ while (my ($schedule_id, $day, $time) = $sched_h->fetchrow_array) {
         # midnight
         my $target_time = ($time =~ /^\d+$/) ? $time : 0;
 
+       my $run_next = $dbh->sql_date_math('(' 
+            . $dbh->sql_date_math('CURRENT_DATE', '+', '?', 'DAY') 
+            . ')', '+', '?', 'HOUR');
         $sth = $dbh->prepare("UPDATE whine_schedules " .
-                             "SET run_next = (CURRENT_DATE + " .
-                             $dbh->sql_interval('?', 'DAY') . ") + " .
-                             $dbh->sql_interval('?', 'HOUR') .
-                             " WHERE id = ?");
+                                "SET run_next = $run_next
+                               WHERE id = ?");
         $sth->execute($target_date, $target_time, $schedule_id);
     }
 }
@@ -349,6 +338,8 @@ while (my $event = get_next_event) {
 #  - queries        array of hashes containing:
 #          - bugs:  array of hashes mapping fieldnames to values for this bug
 #          - title: text title given to this query in the whine event
+#          - columnlist: array of fieldnames to display in the mail
+#          - name:  text name of this query
 #  - schedule_id    integer id of the schedule being run
 #  - subject        Subject line for the message
 #  - recipient      user object for the recipient
@@ -364,7 +355,7 @@ sub mail {
     # Don't send mail to someone whose bugmail notification is disabled.
     return if $addressee->email_disabled;
 
-    my $template = Bugzilla->template_inner($addressee->settings->{'lang'}->{'value'});
+    my $template = Bugzilla->template_inner($addressee->setting('lang'));
     my $msg = ''; # it's a temporary variable to hold the template output
     $args->{'alternatives'} ||= [];
 
@@ -395,7 +386,6 @@ sub mail {
     $template->process("whine/multipart-mime.txt.tmpl", $args, \$msg)
         or die($template->error());
 
-    Bugzilla->template_inner("");
     MessageToMTA($msg);
 
     delete $args->{'boundary'};
@@ -419,6 +409,7 @@ sub run_queries {
               'name'          => $_->[0],
               'title'         => $_->[1],
               'onemailperbug' => $_->[2],
+              'columnlist'    => [],
               'bugs'          => [],
             }
         );
@@ -445,21 +436,37 @@ sub run_queries {
         # Bugzilla::Search to execute a saved query.  It's exceedingly weird,
         # but that's how it works.
         my $searchparams = new Bugzilla::CGI($savedquery);
+
+        # Use the columnlist for the saved query, if it exists, and make
+        # sure bug_id is always in the list.
+        if (my $columnlist = $searchparams->param('columnlist')) {
+            @searchfields = split(/[\s,]+/, $columnlist);
+            unshift(@searchfields, 'bug_id') unless grep { $_ eq 'bug_id' } @searchfields;
+        }
+        push @{$thisquery->{'columnlist'}}, @searchfields;
+
+        my @orderstrings = split(/,\s*/, $searchparams->param('order') || '');
         my $search = new Bugzilla::Search(
             'fields' => \@searchfields,
-            'params' => $searchparams,
+            'params' => scalar $searchparams->Vars,
             'user'   => $args->{'recipient'}, # the search runs as the recipient
+            'order'  => \@orderstrings
         );
-        my $sqlquery = $search->getSQL();
-        $sth = $dbh->prepare($sqlquery);
-        $sth->execute;
+        # If a query fails for whatever reason, it shouldn't kill the script.
+        my $data = eval { $search->data };
+        if ($@) {
+            print STDERR get_text('whine_query_failed', { query_name => $thisquery->{'name'},
+                                                          author => $args->{'author'},
+                                                          reason => $@ }) . "\n";
+            next;
+        }
 
-        while (my @row = $sth->fetchrow_array) {
+        foreach my $row (@$data) {
             my $bug = {};
             for my $field (@searchfields) {
                 my $fieldname = $field;
                 $fieldname =~ s/^bugs\.//;  # No need for bugs.whatever
-                $bug->{$fieldname} = shift @row;
+                $bug->{$fieldname} = shift @$row;
             }
 
             if ($thisquery->{'onemailperbug'}) {
@@ -467,6 +474,7 @@ sub run_queries {
                     {
                         'name' => $thisquery->{'name'},
                         'title' => $thisquery->{'title'},
+                        'columnlist' => $thisquery->{'columnlist'},
                         'bugs' => [ $bug ],
                     },
                 ];
@@ -585,21 +593,22 @@ sub reset_timer {
         my $target_time = ($run_time =~ /^\d+$/) ? $run_time : 0;
 
         my $nextdate = &get_next_date($run_day);
-
+        my $run_next = $dbh->sql_date_math('('
+            . $dbh->sql_date_math('CURRENT_DATE', '+', '?', 'DAY')
+            . ')', '+', '?', 'HOUR');
         $sth = $dbh->prepare("UPDATE whine_schedules " .
-                             "SET run_next = (CURRENT_DATE + " .
-                             $dbh->sql_interval('?', 'DAY') . ") + " .
-                             $dbh->sql_interval('?', 'HOUR') .
-                             " WHERE id = ?");
+                                "SET run_next = $run_next
+                               WHERE id = ?");
         $sth->execute($nextdate, $target_time, $schedule_id);
         return;
     }
 
     if ($minute_offset > 0) {
         # Scheduling is done in terms of whole minutes.
-        my $next_run = $dbh->selectrow_array('SELECT NOW() + ' .
-                                             $dbh->sql_interval('?', 'MINUTE'),
-                                             undef, $minute_offset);
+        
+        my $next_run = $dbh->selectrow_array(
+            'SELECT ' . $dbh->sql_date_math('NOW()', '+', '?', 'MINUTE'),
+            undef, $minute_offset);
         $next_run = format_time($next_run, "%Y-%m-%d %R");
 
         $sth = $dbh->prepare("UPDATE whine_schedules " .
