@@ -1,32 +1,18 @@
 #!/usr/bin/perl -w
-# -*- Mode: perl; indent-tabs-mode: nil -*-
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at http://mozilla.org/MPL/2.0/.
 #
-# The contents of this file are subject to the Mozilla Public
-# License Version 1.1 (the "License"); you may not use this file
-# except in compliance with the License. You may obtain a copy of
-# the License at http://www.mozilla.org/MPL/
-#
-# Software distributed under the License is distributed on an "AS
-# IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
-# implied. See the License for the specific language governing
-# rights and limitations under the License.
-#
-# The Original Code is the Bugzilla Bug Tracking System.
-#
-# The Initial Developer of the Original Code is Netscape Communications
-# Corporation. Portions created by Netscape are
-# Copyright (C) 1998 Netscape Communications Corporation. All
-# Rights Reserved.
-#
-# Contributor(s): Terry Weissman <terry@mozilla.org>
-#                 Joseph Heenan <joseph@heenan.me.uk>
-#                 Frédéric Buclin <LpSolit@gmail.com>
+# This Source Code Form is "Incompatible With Secondary Licenses", as
+# defined by the Mozilla Public License, v. 2.0.
 
 
 # This is a script suitable for running once a day from a cron job.  It 
 # looks at all the bugs, and sends whiny mail to anyone who has a bug 
-# assigned to them that has status NEW or REOPENED that has not been 
-# touched for more than the number of days specified in the whinedays param.
+# assigned to them that has status CONFIRMED, NEW, or REOPENED that has not
+# been touched for more than the number of days specified in the whinedays
+# param. (We have NEW and REOPENED in there to keep compatibility with old
+# Bugzillas.)
 
 use strict;
 use lib qw(. lib);
@@ -44,7 +30,7 @@ my $query = q{SELECT bug_id, short_desc, login_name
                 FROM bugs
           INNER JOIN profiles
                   ON userid = assigned_to
-               WHERE (bug_status = ? OR bug_status = ?)
+               WHERE bug_status IN (?,?,?)
                  AND disable_mail = 0
                  AND } . $dbh->sql_to_days('NOW()') . " - " .
                        $dbh->sql_to_days('delta_ts') . " > " .
@@ -54,7 +40,8 @@ my $query = q{SELECT bug_id, short_desc, login_name
 my %bugs;
 my %desc;
 
-my $slt_bugs = $dbh->selectall_arrayref($query, undef, 'NEW', 'REOPENED');
+my $slt_bugs = $dbh->selectall_arrayref($query, undef, 'CONFIRMED', 'NEW',
+                                                       'REOPENED');
 
 foreach my $bug (@$slt_bugs) {
     my ($id, $desc, $email) = @$bug;
@@ -85,12 +72,11 @@ foreach my $email (sort (keys %bugs)) {
     $vars->{'bugs'} = \@bugs;
 
     my $msg;
-    my $template = Bugzilla->template_inner($user->settings->{'lang'}->{'value'});
+    my $template = Bugzilla->template_inner($user->setting('lang'));
     $template->process("email/whine.txt.tmpl", $vars, \$msg)
       or die($template->error());
 
-    Bugzilla->template_inner("");
     MessageToMTA($msg);
 
-    print "$email      " . join(" ", @{$bugs{$email}}) . "\n";
+    say "$email      " . join(" ", @{$bugs{$email}});
 }
